import sys
from datetime import datetime
from typing import IO, Dict, Iterable, List, Optional, Union

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

Arn = str
Boolean = bool
ConfigurationProfileType = str
DeploymentStrategyId = str
Description = str
Float = float
GrowthFactor = float
Id = str
Identifier = str
Integer = int
LongName = str
MaxResults = int
MinutesBetween0And24Hours = int
Name = str
NextToken = str
Percentage = float
QueryName = str
RoleArn = str
String = str
StringWithLengthBetween0And32768 = str
StringWithLengthBetween1And2048 = str
StringWithLengthBetween1And255 = str
StringWithLengthBetween1And64 = str
TagKey = str
TagValue = str
Uri = str
Version = str


class ActionPoint(str):
    PRE_CREATE_HOSTED_CONFIGURATION_VERSION = "PRE_CREATE_HOSTED_CONFIGURATION_VERSION"
    PRE_START_DEPLOYMENT = "PRE_START_DEPLOYMENT"
    ON_DEPLOYMENT_START = "ON_DEPLOYMENT_START"
    ON_DEPLOYMENT_STEP = "ON_DEPLOYMENT_STEP"
    ON_DEPLOYMENT_BAKING = "ON_DEPLOYMENT_BAKING"
    ON_DEPLOYMENT_COMPLETE = "ON_DEPLOYMENT_COMPLETE"
    ON_DEPLOYMENT_ROLLED_BACK = "ON_DEPLOYMENT_ROLLED_BACK"


class BadRequestReason(str):
    InvalidConfiguration = "InvalidConfiguration"


class BytesMeasure(str):
    KILOBYTES = "KILOBYTES"


class DeploymentEventType(str):
    PERCENTAGE_UPDATED = "PERCENTAGE_UPDATED"
    ROLLBACK_STARTED = "ROLLBACK_STARTED"
    ROLLBACK_COMPLETED = "ROLLBACK_COMPLETED"
    BAKE_TIME_STARTED = "BAKE_TIME_STARTED"
    DEPLOYMENT_STARTED = "DEPLOYMENT_STARTED"
    DEPLOYMENT_COMPLETED = "DEPLOYMENT_COMPLETED"


class DeploymentState(str):
    BAKING = "BAKING"
    VALIDATING = "VALIDATING"
    DEPLOYING = "DEPLOYING"
    COMPLETE = "COMPLETE"
    ROLLING_BACK = "ROLLING_BACK"
    ROLLED_BACK = "ROLLED_BACK"


class EnvironmentState(str):
    READY_FOR_DEPLOYMENT = "READY_FOR_DEPLOYMENT"
    DEPLOYING = "DEPLOYING"
    ROLLING_BACK = "ROLLING_BACK"
    ROLLED_BACK = "ROLLED_BACK"


class GrowthType(str):
    LINEAR = "LINEAR"
    EXPONENTIAL = "EXPONENTIAL"


class ReplicateTo(str):
    NONE = "NONE"
    SSM_DOCUMENT = "SSM_DOCUMENT"


class TriggeredBy(str):
    USER = "USER"
    APPCONFIG = "APPCONFIG"
    CLOUDWATCH_ALARM = "CLOUDWATCH_ALARM"
    INTERNAL_ERROR = "INTERNAL_ERROR"


class ValidatorType(str):
    JSON_SCHEMA = "JSON_SCHEMA"
    LAMBDA = "LAMBDA"


class InvalidConfigurationDetail(TypedDict, total=False):
    """Detailed information about the bad request exception error when creating
    a hosted configuration version.
    """

    Constraint: Optional[String]
    Location: Optional[String]
    Reason: Optional[String]
    Type: Optional[String]
    Value: Optional[String]


InvalidConfigurationDetailList = List[InvalidConfigurationDetail]


class BadRequestDetails(TypedDict, total=False):
    """Detailed information about the input that failed to satisfy the
    constraints specified by a call.
    """

    InvalidConfiguration: Optional[InvalidConfigurationDetailList]


class BadRequestException(ServiceException):
    """The input fails to satisfy the constraints specified by an Amazon Web
    Services service.
    """

    code: str = "BadRequestException"
    sender_fault: bool = False
    status_code: int = 400
    Reason: Optional[BadRequestReason]
    Details: Optional[BadRequestDetails]


class ConflictException(ServiceException):
    """The request could not be processed because of conflict in the current
    state of the resource.
    """

    code: str = "ConflictException"
    sender_fault: bool = False
    status_code: int = 409


class InternalServerException(ServiceException):
    """There was an internal failure in the AppConfig service."""

    code: str = "InternalServerException"
    sender_fault: bool = False
    status_code: int = 500


class PayloadTooLargeException(ServiceException):
    """The configuration size is too large."""

    code: str = "PayloadTooLargeException"
    sender_fault: bool = False
    status_code: int = 413
    Measure: Optional[BytesMeasure]
    Limit: Optional[Float]
    Size: Optional[Float]


class ResourceNotFoundException(ServiceException):
    """The requested resource could not be found."""

    code: str = "ResourceNotFoundException"
    sender_fault: bool = False
    status_code: int = 404
    ResourceName: Optional[String]


class ServiceQuotaExceededException(ServiceException):
    """The number of hosted configuration versions exceeds the limit for the
    AppConfig hosted configuration store. Delete one or more versions and
    try again.
    """

    code: str = "ServiceQuotaExceededException"
    sender_fault: bool = False
    status_code: int = 402


class Action(TypedDict, total=False):
    """An action defines the tasks the extension performs during the AppConfig
    workflow. Each action includes an action point such as
    ``ON_CREATE_HOSTED_CONFIGURATION``, ``PRE_DEPLOYMENT``, or
    ``ON_DEPLOYMENT``. Each action also includes a name, a URI to an Lambda
    function, and an Amazon Resource Name (ARN) for an Identity and Access
    Management assume role. You specify the name, URI, and ARN for each
    *action point* defined in the extension. You can specify the following
    actions for an extension:

    -  ``PRE_CREATE_HOSTED_CONFIGURATION_VERSION``

    -  ``PRE_START_DEPLOYMENT``

    -  ``ON_DEPLOYMENT_START``

    -  ``ON_DEPLOYMENT_STEP``

    -  ``ON_DEPLOYMENT_BAKING``

    -  ``ON_DEPLOYMENT_COMPLETE``

    -  ``ON_DEPLOYMENT_ROLLED_BACK``
    """

    Name: Optional[Name]
    Description: Optional[Description]
    Uri: Optional[Uri]
    RoleArn: Optional[Arn]


class ActionInvocation(TypedDict, total=False):
    """An extension that was invoked as part of a deployment event."""

    ExtensionIdentifier: Optional[Identifier]
    ActionName: Optional[Name]
    Uri: Optional[Uri]
    RoleArn: Optional[Arn]
    ErrorMessage: Optional[String]
    ErrorCode: Optional[String]
    InvocationId: Optional[Id]


ActionInvocations = List[ActionInvocation]
ActionList = List[Action]
ActionsMap = Dict[ActionPoint, ActionList]


class Application(TypedDict, total=False):
    Id: Optional[Id]
    Name: Optional[Name]
    Description: Optional[Description]


ApplicationList = List[Application]


class Applications(TypedDict, total=False):
    Items: Optional[ApplicationList]
    NextToken: Optional[NextToken]


ParameterValueMap = Dict[Name, StringWithLengthBetween1And2048]


class AppliedExtension(TypedDict, total=False):
    """An extension that was invoked during a deployment."""

    ExtensionId: Optional[Id]
    ExtensionAssociationId: Optional[Id]
    VersionNumber: Optional[Integer]
    Parameters: Optional[ParameterValueMap]


AppliedExtensions = List[AppliedExtension]
Blob = bytes


class Configuration(TypedDict, total=False):
    Content: Optional[Union[Blob, IO[Blob], Iterable[Blob]]]
    ConfigurationVersion: Optional[Version]
    ContentType: Optional[String]


class Validator(TypedDict, total=False):
    """A validator provides a syntactic or semantic check to ensure the
    configuration that you want to deploy functions as intended. To validate
    your application configuration data, you provide a schema or an Amazon
    Web Services Lambda function that runs against the configuration. The
    configuration deployment or update can only proceed when the
    configuration data is valid.
    """

    Type: ValidatorType
    Content: StringWithLengthBetween0And32768


ValidatorList = List[Validator]


class ConfigurationProfile(TypedDict, total=False):
    ApplicationId: Optional[Id]
    Id: Optional[Id]
    Name: Optional[LongName]
    Description: Optional[Description]
    LocationUri: Optional[Uri]
    RetrievalRoleArn: Optional[RoleArn]
    Validators: Optional[ValidatorList]
    Type: Optional[ConfigurationProfileType]


ValidatorTypeList = List[ValidatorType]


class ConfigurationProfileSummary(TypedDict, total=False):
    """A summary of a configuration profile."""

    ApplicationId: Optional[Id]
    Id: Optional[Id]
    Name: Optional[LongName]
    LocationUri: Optional[Uri]
    ValidatorTypes: Optional[ValidatorTypeList]
    Type: Optional[ConfigurationProfileType]


ConfigurationProfileSummaryList = List[ConfigurationProfileSummary]


class ConfigurationProfiles(TypedDict, total=False):
    Items: Optional[ConfigurationProfileSummaryList]
    NextToken: Optional[NextToken]


TagMap = Dict[TagKey, TagValue]


class CreateApplicationRequest(ServiceRequest):
    Name: Name
    Description: Optional[Description]
    Tags: Optional[TagMap]


class CreateConfigurationProfileRequest(ServiceRequest):
    ApplicationId: Id
    Name: LongName
    Description: Optional[Description]
    LocationUri: Uri
    RetrievalRoleArn: Optional[RoleArn]
    Validators: Optional[ValidatorList]
    Tags: Optional[TagMap]
    Type: Optional[ConfigurationProfileType]


class CreateDeploymentStrategyRequest(ServiceRequest):
    Name: Name
    Description: Optional[Description]
    DeploymentDurationInMinutes: MinutesBetween0And24Hours
    FinalBakeTimeInMinutes: Optional[MinutesBetween0And24Hours]
    GrowthFactor: GrowthFactor
    GrowthType: Optional[GrowthType]
    ReplicateTo: Optional[ReplicateTo]
    Tags: Optional[TagMap]


class Monitor(TypedDict, total=False):
    """Amazon CloudWatch alarms to monitor during the deployment process."""

    AlarmArn: StringWithLengthBetween1And2048
    AlarmRoleArn: Optional[RoleArn]


MonitorList = List[Monitor]


class CreateEnvironmentRequest(ServiceRequest):
    ApplicationId: Id
    Name: Name
    Description: Optional[Description]
    Monitors: Optional[MonitorList]
    Tags: Optional[TagMap]


class CreateExtensionAssociationRequest(ServiceRequest):
    ExtensionIdentifier: Identifier
    ExtensionVersionNumber: Optional[Integer]
    ResourceIdentifier: Identifier
    Parameters: Optional[ParameterValueMap]
    Tags: Optional[TagMap]


class Parameter(TypedDict, total=False):
    """A value such as an Amazon Resource Name (ARN) or an Amazon Simple
    Notification Service topic entered in an extension when invoked.
    Parameter values are specified in an extension association. For more
    information about extensions, see `Working with AppConfig
    extensions <https://docs.aws.amazon.com/appconfig/latest/userguide/working-with-appconfig-extensions.html>`__
    in the *AppConfig User Guide*.
    """

    Description: Optional[Description]
    Required: Optional[Boolean]


ParameterMap = Dict[Name, Parameter]


class CreateExtensionRequest(ServiceRequest):
    Name: Name
    Description: Optional[Description]
    Actions: ActionsMap
    Parameters: Optional[ParameterMap]
    Tags: Optional[TagMap]
    LatestVersionNumber: Optional[Integer]


class CreateHostedConfigurationVersionRequest(ServiceRequest):
    Content: IO[Blob]
    ApplicationId: Id
    ConfigurationProfileId: Id
    Description: Optional[Description]
    ContentType: StringWithLengthBetween1And255
    LatestVersionNumber: Optional[Integer]


class DeleteApplicationRequest(ServiceRequest):
    ApplicationId: Id


class DeleteConfigurationProfileRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id


class DeleteDeploymentStrategyRequest(ServiceRequest):
    DeploymentStrategyId: DeploymentStrategyId


class DeleteEnvironmentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id


class DeleteExtensionAssociationRequest(ServiceRequest):
    ExtensionAssociationId: Id


class DeleteExtensionRequest(ServiceRequest):
    ExtensionIdentifier: Identifier
    VersionNumber: Optional[Integer]


class DeleteHostedConfigurationVersionRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id
    VersionNumber: Integer


Iso8601DateTime = datetime


class DeploymentEvent(TypedDict, total=False):
    """An object that describes a deployment event."""

    EventType: Optional[DeploymentEventType]
    TriggeredBy: Optional[TriggeredBy]
    Description: Optional[Description]
    ActionInvocations: Optional[ActionInvocations]
    OccurredAt: Optional[Iso8601DateTime]


DeploymentEvents = List[DeploymentEvent]


class Deployment(TypedDict, total=False):
    ApplicationId: Optional[Id]
    EnvironmentId: Optional[Id]
    DeploymentStrategyId: Optional[Id]
    ConfigurationProfileId: Optional[Id]
    DeploymentNumber: Optional[Integer]
    ConfigurationName: Optional[Name]
    ConfigurationLocationUri: Optional[Uri]
    ConfigurationVersion: Optional[Version]
    Description: Optional[Description]
    DeploymentDurationInMinutes: Optional[MinutesBetween0And24Hours]
    GrowthType: Optional[GrowthType]
    GrowthFactor: Optional[Percentage]
    FinalBakeTimeInMinutes: Optional[MinutesBetween0And24Hours]
    State: Optional[DeploymentState]
    EventLog: Optional[DeploymentEvents]
    PercentageComplete: Optional[Percentage]
    StartedAt: Optional[Iso8601DateTime]
    CompletedAt: Optional[Iso8601DateTime]
    AppliedExtensions: Optional[AppliedExtensions]


class DeploymentSummary(TypedDict, total=False):
    """Information about the deployment."""

    DeploymentNumber: Optional[Integer]
    ConfigurationName: Optional[Name]
    ConfigurationVersion: Optional[Version]
    DeploymentDurationInMinutes: Optional[MinutesBetween0And24Hours]
    GrowthType: Optional[GrowthType]
    GrowthFactor: Optional[Percentage]
    FinalBakeTimeInMinutes: Optional[MinutesBetween0And24Hours]
    State: Optional[DeploymentState]
    PercentageComplete: Optional[Percentage]
    StartedAt: Optional[Iso8601DateTime]
    CompletedAt: Optional[Iso8601DateTime]


DeploymentList = List[DeploymentSummary]


class DeploymentStrategy(TypedDict, total=False):
    Id: Optional[Id]
    Name: Optional[Name]
    Description: Optional[Description]
    DeploymentDurationInMinutes: Optional[MinutesBetween0And24Hours]
    GrowthType: Optional[GrowthType]
    GrowthFactor: Optional[Percentage]
    FinalBakeTimeInMinutes: Optional[MinutesBetween0And24Hours]
    ReplicateTo: Optional[ReplicateTo]


DeploymentStrategyList = List[DeploymentStrategy]


class DeploymentStrategies(TypedDict, total=False):
    Items: Optional[DeploymentStrategyList]
    NextToken: Optional[NextToken]


class Deployments(TypedDict, total=False):
    Items: Optional[DeploymentList]
    NextToken: Optional[NextToken]


class Environment(TypedDict, total=False):
    ApplicationId: Optional[Id]
    Id: Optional[Id]
    Name: Optional[Name]
    Description: Optional[Description]
    State: Optional[EnvironmentState]
    Monitors: Optional[MonitorList]


EnvironmentList = List[Environment]


class Environments(TypedDict, total=False):
    Items: Optional[EnvironmentList]
    NextToken: Optional[NextToken]


class Extension(TypedDict, total=False):
    Id: Optional[Id]
    Name: Optional[Name]
    VersionNumber: Optional[Integer]
    Arn: Optional[Arn]
    Description: Optional[Description]
    Actions: Optional[ActionsMap]
    Parameters: Optional[ParameterMap]


class ExtensionAssociation(TypedDict, total=False):
    Id: Optional[Identifier]
    ExtensionArn: Optional[Arn]
    ResourceArn: Optional[Arn]
    Arn: Optional[Arn]
    Parameters: Optional[ParameterValueMap]
    ExtensionVersionNumber: Optional[Integer]


class ExtensionAssociationSummary(TypedDict, total=False):
    """Information about an association between an extension and an AppConfig
    resource such as an application, environment, or configuration profile.
    Call ``GetExtensionAssociation`` to get more information about an
    association.
    """

    Id: Optional[Identifier]
    ExtensionArn: Optional[Arn]
    ResourceArn: Optional[Arn]


ExtensionAssociationSummaries = List[ExtensionAssociationSummary]


class ExtensionAssociations(TypedDict, total=False):
    Items: Optional[ExtensionAssociationSummaries]
    NextToken: Optional[NextToken]


class ExtensionSummary(TypedDict, total=False):
    """Information about an extension. Call ``GetExtension`` to get more
    information about an extension.
    """

    Id: Optional[Id]
    Name: Optional[Name]
    VersionNumber: Optional[Integer]
    Arn: Optional[Arn]
    Description: Optional[Description]


ExtensionSummaries = List[ExtensionSummary]


class Extensions(TypedDict, total=False):
    Items: Optional[ExtensionSummaries]
    NextToken: Optional[NextToken]


class GetApplicationRequest(ServiceRequest):
    ApplicationId: Id


class GetConfigurationProfileRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id


class GetConfigurationRequest(ServiceRequest):
    Application: StringWithLengthBetween1And64
    Environment: StringWithLengthBetween1And64
    Configuration: StringWithLengthBetween1And64
    ClientId: StringWithLengthBetween1And64
    ClientConfigurationVersion: Optional[Version]


class GetDeploymentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id
    DeploymentNumber: Integer


class GetDeploymentStrategyRequest(ServiceRequest):
    DeploymentStrategyId: DeploymentStrategyId


class GetEnvironmentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id


class GetExtensionAssociationRequest(ServiceRequest):
    ExtensionAssociationId: Id


class GetExtensionRequest(ServiceRequest):
    ExtensionIdentifier: Identifier
    VersionNumber: Optional[Integer]


class GetHostedConfigurationVersionRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id
    VersionNumber: Integer


class HostedConfigurationVersion(TypedDict, total=False):
    Content: Optional[Union[Blob, IO[Blob], Iterable[Blob]]]
    ApplicationId: Optional[Id]
    ConfigurationProfileId: Optional[Id]
    VersionNumber: Optional[Integer]
    Description: Optional[Description]
    ContentType: Optional[StringWithLengthBetween1And255]


class HostedConfigurationVersionSummary(TypedDict, total=False):
    """Information about the configuration."""

    ApplicationId: Optional[Id]
    ConfigurationProfileId: Optional[Id]
    VersionNumber: Optional[Integer]
    Description: Optional[Description]
    ContentType: Optional[StringWithLengthBetween1And255]


HostedConfigurationVersionSummaryList = List[HostedConfigurationVersionSummary]


class HostedConfigurationVersions(TypedDict, total=False):
    Items: Optional[HostedConfigurationVersionSummaryList]
    NextToken: Optional[NextToken]


class ListApplicationsRequest(ServiceRequest):
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]


class ListConfigurationProfilesRequest(ServiceRequest):
    ApplicationId: Id
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]
    Type: Optional[ConfigurationProfileType]


class ListDeploymentStrategiesRequest(ServiceRequest):
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]


class ListDeploymentsRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]


class ListEnvironmentsRequest(ServiceRequest):
    ApplicationId: Id
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]


class ListExtensionAssociationsRequest(ServiceRequest):
    ResourceIdentifier: Optional[Arn]
    ExtensionIdentifier: Optional[Identifier]
    ExtensionVersionNumber: Optional[Integer]
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]


class ListExtensionsRequest(ServiceRequest):
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]
    Name: Optional[QueryName]


class ListHostedConfigurationVersionsRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id
    MaxResults: Optional[MaxResults]
    NextToken: Optional[NextToken]


class ListTagsForResourceRequest(ServiceRequest):
    ResourceArn: Arn


class ResourceTags(TypedDict, total=False):
    Tags: Optional[TagMap]


class StartDeploymentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id
    DeploymentStrategyId: DeploymentStrategyId
    ConfigurationProfileId: Id
    ConfigurationVersion: Version
    Description: Optional[Description]
    Tags: Optional[TagMap]


class StopDeploymentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id
    DeploymentNumber: Integer


TagKeyList = List[TagKey]


class TagResourceRequest(ServiceRequest):
    ResourceArn: Arn
    Tags: TagMap


class UntagResourceRequest(ServiceRequest):
    ResourceArn: Arn
    TagKeys: TagKeyList


class UpdateApplicationRequest(ServiceRequest):
    ApplicationId: Id
    Name: Optional[Name]
    Description: Optional[Description]


class UpdateConfigurationProfileRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id
    Name: Optional[Name]
    Description: Optional[Description]
    RetrievalRoleArn: Optional[RoleArn]
    Validators: Optional[ValidatorList]


class UpdateDeploymentStrategyRequest(ServiceRequest):
    DeploymentStrategyId: DeploymentStrategyId
    Description: Optional[Description]
    DeploymentDurationInMinutes: Optional[MinutesBetween0And24Hours]
    FinalBakeTimeInMinutes: Optional[MinutesBetween0And24Hours]
    GrowthFactor: Optional[GrowthFactor]
    GrowthType: Optional[GrowthType]


class UpdateEnvironmentRequest(ServiceRequest):
    ApplicationId: Id
    EnvironmentId: Id
    Name: Optional[Name]
    Description: Optional[Description]
    Monitors: Optional[MonitorList]


class UpdateExtensionAssociationRequest(ServiceRequest):
    ExtensionAssociationId: Id
    Parameters: Optional[ParameterValueMap]


class UpdateExtensionRequest(ServiceRequest):
    ExtensionIdentifier: Identifier
    Description: Optional[Description]
    Actions: Optional[ActionsMap]
    Parameters: Optional[ParameterMap]
    VersionNumber: Optional[Integer]


class ValidateConfigurationRequest(ServiceRequest):
    ApplicationId: Id
    ConfigurationProfileId: Id
    ConfigurationVersion: Version


class AppconfigApi:

    service = "appconfig"
    version = "2019-10-09"

    @handler("CreateApplication")
    def create_application(
        self,
        context: RequestContext,
        name: Name,
        description: Description = None,
        tags: TagMap = None,
    ) -> Application:
        """Creates an application. In AppConfig, an application is simply an
        organizational construct like a folder. This organizational construct
        has a relationship with some unit of executable code. For example, you
        could create an application called MyMobileApp to organize and manage
        configuration data for a mobile application installed by your users.

        :param name: A name for the application.
        :param description: A description of the application.
        :param tags: Metadata to assign to the application.
        :returns: Application
        :raises BadRequestException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("CreateConfigurationProfile", expand=False)
    def create_configuration_profile(
        self, context: RequestContext, request: CreateConfigurationProfileRequest
    ) -> ConfigurationProfile:
        """Creates a configuration profile, which is information that enables
        AppConfig to access the configuration source. Valid configuration
        sources include the AppConfig hosted configuration store, Amazon Web
        Services Systems Manager (SSM) documents, SSM Parameter Store
        parameters, Amazon S3 objects, or any `integration source
        action <http://docs.aws.amazon.com/codepipeline/latest/userguide/integrations-action-type.html#integrations-source>`__
        supported by CodePipeline. A configuration profile includes the
        following information:

        -  The URI location of the configuration data.

        -  The Identity and Access Management (IAM) role that provides access to
           the configuration data.

        -  A validator for the configuration data. Available validators include
           either a JSON Schema or an Amazon Web Services Lambda function.

        For more information, see `Create a Configuration and a Configuration
        Profile <http://docs.aws.amazon.com/appconfig/latest/userguide/appconfig-creating-configuration-and-profile.html>`__
        in the *AppConfig User Guide*.

        :param application_id: The application ID.
        :param name: A name for the configuration profile.
        :param location_uri: A URI to locate the configuration.
        :param description: A description of the configuration profile.
        :param retrieval_role_arn: The ARN of an IAM role with permission to access the configuration at
        the specified ``LocationUri``.
        :param validators: A list of methods for validating the configuration.
        :param tags: Metadata to assign to the configuration profile.
        :param type: The type of configurations contained in the profile.
        :returns: ConfigurationProfile
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("CreateDeploymentStrategy")
    def create_deployment_strategy(
        self,
        context: RequestContext,
        name: Name,
        deployment_duration_in_minutes: MinutesBetween0And24Hours,
        growth_factor: GrowthFactor,
        description: Description = None,
        final_bake_time_in_minutes: MinutesBetween0And24Hours = None,
        growth_type: GrowthType = None,
        replicate_to: ReplicateTo = None,
        tags: TagMap = None,
    ) -> DeploymentStrategy:
        """Creates a deployment strategy that defines important criteria for
        rolling out your configuration to the designated targets. A deployment
        strategy includes the overall duration required, a percentage of targets
        to receive the deployment during each interval, an algorithm that
        defines how percentage grows, and bake time.

        :param name: A name for the deployment strategy.
        :param deployment_duration_in_minutes: Total amount of time for a deployment to last.
        :param growth_factor: The percentage of targets to receive a deployed configuration during
        each interval.
        :param description: A description of the deployment strategy.
        :param final_bake_time_in_minutes: Specifies the amount of time AppConfig monitors for Amazon CloudWatch
        alarms after the configuration has been deployed to 100% of its targets,
        before considering the deployment to be complete.
        :param growth_type: The algorithm used to define how percentage grows over time.
        :param replicate_to: Save the deployment strategy to a Systems Manager (SSM) document.
        :param tags: Metadata to assign to the deployment strategy.
        :returns: DeploymentStrategy
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("CreateEnvironment")
    def create_environment(
        self,
        context: RequestContext,
        application_id: Id,
        name: Name,
        description: Description = None,
        monitors: MonitorList = None,
        tags: TagMap = None,
    ) -> Environment:
        """Creates an environment. For each application, you define one or more
        environments. An environment is a deployment group of AppConfig targets,
        such as applications in a ``Beta`` or ``Production`` environment. You
        can also define environments for application subcomponents such as the
        ``Web``, ``Mobile`` and ``Back-end`` components for your application.
        You can configure Amazon CloudWatch alarms for each environment. The
        system monitors alarms during a configuration deployment. If an alarm is
        triggered, the system rolls back the configuration.

        :param application_id: The application ID.
        :param name: A name for the environment.
        :param description: A description of the environment.
        :param monitors: Amazon CloudWatch alarms to monitor during the deployment process.
        :param tags: Metadata to assign to the environment.
        :returns: Environment
        :raises InternalServerException:
        :raises ResourceNotFoundException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("CreateExtension")
    def create_extension(
        self,
        context: RequestContext,
        name: Name,
        actions: ActionsMap,
        description: Description = None,
        parameters: ParameterMap = None,
        tags: TagMap = None,
        latest_version_number: Integer = None,
    ) -> Extension:
        """Creates an AppConfig extension. An extension augments your ability to
        inject logic or behavior at different points during the AppConfig
        workflow of creating or deploying a configuration.

        You can create your own extensions or use the Amazon Web
        Services-authored extensions provided by AppConfig. For most use-cases,
        to create your own extension, you must create an Lambda function to
        perform any computation and processing defined in the extension. For
        more information about extensions, see `Working with AppConfig
        extensions <https://docs.aws.amazon.com/appconfig/latest/userguide/working-with-appconfig-extensions.html>`__
        in the *AppConfig User Guide*.

        :param name: A name for the extension.
        :param actions: The actions defined in the extension.
        :param description: Information about the extension.
        :param parameters: The parameters accepted by the extension.
        :param tags: Adds one or more tags for the specified extension.
        :param latest_version_number: You can omit this field when you create an extension.
        :returns: Extension
        :raises BadRequestException:
        :raises ConflictException:
        :raises ServiceQuotaExceededException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("CreateExtensionAssociation")
    def create_extension_association(
        self,
        context: RequestContext,
        extension_identifier: Identifier,
        resource_identifier: Identifier,
        extension_version_number: Integer = None,
        parameters: ParameterValueMap = None,
        tags: TagMap = None,
    ) -> ExtensionAssociation:
        """When you create an extension or configure an Amazon Web
        Services-authored extension, you associate the extension with an
        AppConfig application, environment, or configuration profile. For
        example, you can choose to run the
        ``AppConfig deployment events to Amazon SNS`` Amazon Web
        Services-authored extension and receive notifications on an Amazon SNS
        topic anytime a configuration deployment is started for a specific
        application. Defining which extension to associate with an AppConfig
        resource is called an *extension association*. An extension association
        is a specified relationship between an extension and an AppConfig
        resource, such as an application or a configuration profile. For more
        information about extensions and associations, see `Working with
        AppConfig
        extensions <https://docs.aws.amazon.com/appconfig/latest/userguide/working-with-appconfig-extensions.html>`__
        in the *AppConfig User Guide*.

        :param extension_identifier: The name, the ID, or the Amazon Resource Name (ARN) of the extension.
        :param resource_identifier: The ARN of an application, configuration profile, or environment.
        :param extension_version_number: The version number of the extension.
        :param parameters: The parameter names and values defined in the extensions.
        :param tags: Adds one or more tags for the specified extension association.
        :returns: ExtensionAssociation
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises ServiceQuotaExceededException:
        """
        raise NotImplementedError

    @handler("CreateHostedConfigurationVersion")
    def create_hosted_configuration_version(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        content: IO[Blob],
        content_type: StringWithLengthBetween1And255,
        description: Description = None,
        latest_version_number: Integer = None,
    ) -> HostedConfigurationVersion:
        """Creates a new configuration in the AppConfig hosted configuration store.

        :param application_id: The application ID.
        :param configuration_profile_id: The configuration profile ID.
        :param content: The content of the configuration or the configuration data.
        :param content_type: A standard MIME type describing the format of the configuration content.
        :param description: A description of the configuration.
        :param latest_version_number: An optional locking token used to prevent race conditions from
        overwriting configuration updates when creating a new version.
        :returns: HostedConfigurationVersion
        :raises BadRequestException:
        :raises ServiceQuotaExceededException:
        :raises ResourceNotFoundException:
        :raises ConflictException:
        :raises PayloadTooLargeException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("DeleteApplication")
    def delete_application(self, context: RequestContext, application_id: Id) -> None:
        """Deletes an application. Deleting an application does not delete a
        configuration from a host.

        :param application_id: The ID of the application to delete.
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("DeleteConfigurationProfile")
    def delete_configuration_profile(
        self, context: RequestContext, application_id: Id, configuration_profile_id: Id
    ) -> None:
        """Deletes a configuration profile. Deleting a configuration profile does
        not delete a configuration from a host.

        :param application_id: The application ID that includes the configuration profile you want to
        delete.
        :param configuration_profile_id: The ID of the configuration profile you want to delete.
        :raises ResourceNotFoundException:
        :raises ConflictException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("DeleteDeploymentStrategy")
    def delete_deployment_strategy(
        self, context: RequestContext, deployment_strategy_id: DeploymentStrategyId
    ) -> None:
        """Deletes a deployment strategy. Deleting a deployment strategy does not
        delete a configuration from a host.

        :param deployment_strategy_id: The ID of the deployment strategy you want to delete.
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("DeleteEnvironment")
    def delete_environment(
        self, context: RequestContext, application_id: Id, environment_id: Id
    ) -> None:
        """Deletes an environment. Deleting an environment does not delete a
        configuration from a host.

        :param application_id: The application ID that includes the environment that you want to
        delete.
        :param environment_id: The ID of the environment that you want to delete.
        :raises ResourceNotFoundException:
        :raises ConflictException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("DeleteExtension")
    def delete_extension(
        self,
        context: RequestContext,
        extension_identifier: Identifier,
        version_number: Integer = None,
    ) -> None:
        """Deletes an AppConfig extension. You must delete all associations to an
        extension before you delete the extension.

        :param extension_identifier: The name, ID, or Amazon Resource Name (ARN) of the extension you want to
        delete.
        :param version_number: A specific version of an extension to delete.
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("DeleteExtensionAssociation")
    def delete_extension_association(
        self, context: RequestContext, extension_association_id: Id
    ) -> None:
        """Deletes an extension association. This action doesn't delete extensions
        defined in the association.

        :param extension_association_id: The ID of the extension association to delete.
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("DeleteHostedConfigurationVersion")
    def delete_hosted_configuration_version(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        version_number: Integer,
    ) -> None:
        """Deletes a version of a configuration from the AppConfig hosted
        configuration store.

        :param application_id: The application ID.
        :param configuration_profile_id: The configuration profile ID.
        :param version_number: The versions number to delete.
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("GetApplication")
    def get_application(self, context: RequestContext, application_id: Id) -> Application:
        """Retrieves information about an application.

        :param application_id: The ID of the application you want to get.
        :returns: Application
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetConfiguration")
    def get_configuration(
        self,
        context: RequestContext,
        application: StringWithLengthBetween1And64,
        environment: StringWithLengthBetween1And64,
        configuration: StringWithLengthBetween1And64,
        client_id: StringWithLengthBetween1And64,
        client_configuration_version: Version = None,
    ) -> Configuration:
        """Retrieves the latest deployed configuration.

        Note the following important information.

        -  This API action has been deprecated. Calls to receive configuration
           data should use the
           `StartConfigurationSession <https://docs.aws.amazon.com/appconfig/2019-10-09/APIReference/API_appconfigdata_StartConfigurationSession.html>`__
           and
           `GetLatestConfiguration <https://docs.aws.amazon.com/appconfig/2019-10-09/APIReference/API_appconfigdata_GetLatestConfiguration.html>`__
           APIs instead.

        -  ``GetConfiguration`` is a priced call. For more information, see
           `Pricing <https://aws.amazon.com/systems-manager/pricing/>`__.

        -  AppConfig uses the value of the ``ClientConfigurationVersion``
           parameter to identify the configuration version on your clients. If
           you don’t send ``ClientConfigurationVersion`` with each call to
           ``GetConfiguration``, your clients receive the current configuration.
           You are charged each time your clients receive a configuration.

           To avoid excess charges, we recommend you use the
           `StartConfigurationSession <https://docs.aws.amazon.com/appconfig/2019-10-09/APIReference/StartConfigurationSession.html>`__
           and
           `GetLatestConfiguration <https://docs.aws.amazon.com/appconfig/2019-10-09/APIReference/GetLatestConfiguration.html>`__
           APIs, which track the client configuration version on your behalf. If
           you choose to continue using ``GetConfiguration``, we recommend that
           you include the ``ClientConfigurationVersion`` value with every call
           to ``GetConfiguration``. The value to use for
           ``ClientConfigurationVersion`` comes from the
           ``ConfigurationVersion`` attribute returned by ``GetConfiguration``
           when there is new or updated data, and should be saved for subsequent
           calls to ``GetConfiguration``.

        :param application: The application to get.
        :param environment: The environment to get.
        :param configuration: The configuration to get.
        :param client_id: The clientId parameter in the following command is a unique,
        user-specified ID to identify the client for the configuration.
        :param client_configuration_version: The configuration version returned in the most recent
        ``GetConfiguration`` response.
        :returns: Configuration
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetConfigurationProfile")
    def get_configuration_profile(
        self, context: RequestContext, application_id: Id, configuration_profile_id: Id
    ) -> ConfigurationProfile:
        """Retrieves information about a configuration profile.

        :param application_id: The ID of the application that includes the configuration profile you
        want to get.
        :param configuration_profile_id: The ID of the configuration profile that you want to get.
        :returns: ConfigurationProfile
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetDeployment")
    def get_deployment(
        self,
        context: RequestContext,
        application_id: Id,
        environment_id: Id,
        deployment_number: Integer,
    ) -> Deployment:
        """Retrieves information about a configuration deployment.

        :param application_id: The ID of the application that includes the deployment you want to get.
        :param environment_id: The ID of the environment that includes the deployment you want to get.
        :param deployment_number: The sequence number of the deployment.
        :returns: Deployment
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetDeploymentStrategy")
    def get_deployment_strategy(
        self, context: RequestContext, deployment_strategy_id: DeploymentStrategyId
    ) -> DeploymentStrategy:
        """Retrieves information about a deployment strategy. A deployment strategy
        defines important criteria for rolling out your configuration to the
        designated targets. A deployment strategy includes the overall duration
        required, a percentage of targets to receive the deployment during each
        interval, an algorithm that defines how percentage grows, and bake time.

        :param deployment_strategy_id: The ID of the deployment strategy to get.
        :returns: DeploymentStrategy
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetEnvironment")
    def get_environment(
        self, context: RequestContext, application_id: Id, environment_id: Id
    ) -> Environment:
        """Retrieves information about an environment. An environment is a
        deployment group of AppConfig applications, such as applications in a
        ``Production`` environment or in an ``EU_Region`` environment. Each
        configuration deployment targets an environment. You can enable one or
        more Amazon CloudWatch alarms for an environment. If an alarm is
        triggered during a deployment, AppConfig roles back the configuration.

        :param application_id: The ID of the application that includes the environment you want to get.
        :param environment_id: The ID of the environment that you want to get.
        :returns: Environment
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetExtension")
    def get_extension(
        self,
        context: RequestContext,
        extension_identifier: Identifier,
        version_number: Integer = None,
    ) -> Extension:
        """Returns information about an AppConfig extension.

        :param extension_identifier: The name, the ID, or the Amazon Resource Name (ARN) of the extension.
        :param version_number: The extension version number.
        :returns: Extension
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("GetExtensionAssociation")
    def get_extension_association(
        self, context: RequestContext, extension_association_id: Id
    ) -> ExtensionAssociation:
        """Returns information about an AppConfig extension association. For more
        information about extensions and associations, see `Working with
        AppConfig
        extensions <https://docs.aws.amazon.com/appconfig/latest/userguide/working-with-appconfig-extensions.html>`__
        in the *AppConfig User Guide*.

        :param extension_association_id: The extension association ID to get.
        :returns: ExtensionAssociation
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("GetHostedConfigurationVersion")
    def get_hosted_configuration_version(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        version_number: Integer,
    ) -> HostedConfigurationVersion:
        """Retrieves information about a specific configuration version.

        :param application_id: The application ID.
        :param configuration_profile_id: The configuration profile ID.
        :param version_number: The version.
        :returns: HostedConfigurationVersion
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("ListApplications")
    def list_applications(
        self, context: RequestContext, max_results: MaxResults = None, next_token: NextToken = None
    ) -> Applications:
        """Lists all applications in your Amazon Web Services account.

        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :returns: Applications
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListConfigurationProfiles", expand=False)
    def list_configuration_profiles(
        self, context: RequestContext, request: ListConfigurationProfilesRequest
    ) -> ConfigurationProfiles:
        """Lists the configuration profiles for an application.

        :param application_id: The application ID.
        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :param type: A filter based on the type of configurations that the configuration
        profile contains.
        :returns: ConfigurationProfiles
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListDeploymentStrategies")
    def list_deployment_strategies(
        self, context: RequestContext, max_results: MaxResults = None, next_token: NextToken = None
    ) -> DeploymentStrategies:
        """Lists deployment strategies.

        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :returns: DeploymentStrategies
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListDeployments")
    def list_deployments(
        self,
        context: RequestContext,
        application_id: Id,
        environment_id: Id,
        max_results: MaxResults = None,
        next_token: NextToken = None,
    ) -> Deployments:
        """Lists the deployments for an environment in descending deployment number
        order.

        :param application_id: The application ID.
        :param environment_id: The environment ID.
        :param max_results: The maximum number of items that may be returned for this call.
        :param next_token: The token returned by a prior call to this operation indicating the next
        set of results to be returned.
        :returns: Deployments
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListEnvironments")
    def list_environments(
        self,
        context: RequestContext,
        application_id: Id,
        max_results: MaxResults = None,
        next_token: NextToken = None,
    ) -> Environments:
        """Lists the environments for an application.

        :param application_id: The application ID.
        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :returns: Environments
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListExtensionAssociations")
    def list_extension_associations(
        self,
        context: RequestContext,
        resource_identifier: Arn = None,
        extension_identifier: Identifier = None,
        extension_version_number: Integer = None,
        max_results: MaxResults = None,
        next_token: NextToken = None,
    ) -> ExtensionAssociations:
        """Lists all AppConfig extension associations in the account. For more
        information about extensions and associations, see `Working with
        AppConfig
        extensions <https://docs.aws.amazon.com/appconfig/latest/userguide/working-with-appconfig-extensions.html>`__
        in the *AppConfig User Guide*.

        :param resource_identifier: The ARN of an application, configuration profile, or environment.
        :param extension_identifier: The name, the ID, or the Amazon Resource Name (ARN) of the extension.
        :param extension_version_number: The version number for the extension defined in the association.
        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :returns: ExtensionAssociations
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListExtensions")
    def list_extensions(
        self,
        context: RequestContext,
        max_results: MaxResults = None,
        next_token: NextToken = None,
        name: QueryName = None,
    ) -> Extensions:
        """Lists all custom and Amazon Web Services-authored AppConfig extensions
        in the account. For more information about extensions, see `Working with
        AppConfig
        extensions <https://docs.aws.amazon.com/appconfig/latest/userguide/working-with-appconfig-extensions.html>`__
        in the *AppConfig User Guide*.

        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :param name: The extension name.
        :returns: Extensions
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("ListHostedConfigurationVersions")
    def list_hosted_configuration_versions(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        max_results: MaxResults = None,
        next_token: NextToken = None,
    ) -> HostedConfigurationVersions:
        """Lists configurations stored in the AppConfig hosted configuration store
        by version.

        :param application_id: The application ID.
        :param configuration_profile_id: The configuration profile ID.
        :param max_results: The maximum number of items to return for this call.
        :param next_token: A token to start the list.
        :returns: HostedConfigurationVersions
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("ListTagsForResource")
    def list_tags_for_resource(self, context: RequestContext, resource_arn: Arn) -> ResourceTags:
        """Retrieves the list of key-value tags assigned to the resource.

        :param resource_arn: The resource ARN.
        :returns: ResourceTags
        :raises ResourceNotFoundException:
        :raises BadRequestException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("StartDeployment")
    def start_deployment(
        self,
        context: RequestContext,
        application_id: Id,
        environment_id: Id,
        deployment_strategy_id: DeploymentStrategyId,
        configuration_profile_id: Id,
        configuration_version: Version,
        description: Description = None,
        tags: TagMap = None,
    ) -> Deployment:
        """Starts a deployment.

        :param application_id: The application ID.
        :param environment_id: The environment ID.
        :param deployment_strategy_id: The deployment strategy ID.
        :param configuration_profile_id: The configuration profile ID.
        :param configuration_version: The configuration version to deploy.
        :param description: A description of the deployment.
        :param tags: Metadata to assign to the deployment.
        :returns: Deployment
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises ConflictException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("StopDeployment")
    def stop_deployment(
        self,
        context: RequestContext,
        application_id: Id,
        environment_id: Id,
        deployment_number: Integer,
    ) -> Deployment:
        """Stops a deployment. This API action works only on deployments that have
        a status of ``DEPLOYING``. This action moves the deployment to a status
        of ``ROLLED_BACK``.

        :param application_id: The application ID.
        :param environment_id: The environment ID.
        :param deployment_number: The sequence number of the deployment.
        :returns: Deployment
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        :raises BadRequestException:
        """
        raise NotImplementedError

    @handler("TagResource")
    def tag_resource(self, context: RequestContext, resource_arn: Arn, tags: TagMap) -> None:
        """Assigns metadata to an AppConfig resource. Tags help organize and
        categorize your AppConfig resources. Each tag consists of a key and an
        optional value, both of which you define. You can specify a maximum of
        50 tags for a resource.

        :param resource_arn: The ARN of the resource for which to retrieve tags.
        :param tags: The key-value string map.
        :raises ResourceNotFoundException:
        :raises BadRequestException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UntagResource")
    def untag_resource(
        self, context: RequestContext, resource_arn: Arn, tag_keys: TagKeyList
    ) -> None:
        """Deletes a tag key and value from an AppConfig resource.

        :param resource_arn: The ARN of the resource for which to remove tags.
        :param tag_keys: The tag keys to delete.
        :raises ResourceNotFoundException:
        :raises BadRequestException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UpdateApplication")
    def update_application(
        self,
        context: RequestContext,
        application_id: Id,
        name: Name = None,
        description: Description = None,
    ) -> Application:
        """Updates an application.

        :param application_id: The application ID.
        :param name: The name of the application.
        :param description: A description of the application.
        :returns: Application
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UpdateConfigurationProfile")
    def update_configuration_profile(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        name: Name = None,
        description: Description = None,
        retrieval_role_arn: RoleArn = None,
        validators: ValidatorList = None,
    ) -> ConfigurationProfile:
        """Updates a configuration profile.

        :param application_id: The application ID.
        :param configuration_profile_id: The ID of the configuration profile.
        :param name: The name of the configuration profile.
        :param description: A description of the configuration profile.
        :param retrieval_role_arn: The ARN of an IAM role with permission to access the configuration at
        the specified ``LocationUri``.
        :param validators: A list of methods for validating the configuration.
        :returns: ConfigurationProfile
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UpdateDeploymentStrategy")
    def update_deployment_strategy(
        self,
        context: RequestContext,
        deployment_strategy_id: DeploymentStrategyId,
        description: Description = None,
        deployment_duration_in_minutes: MinutesBetween0And24Hours = None,
        final_bake_time_in_minutes: MinutesBetween0And24Hours = None,
        growth_factor: GrowthFactor = None,
        growth_type: GrowthType = None,
    ) -> DeploymentStrategy:
        """Updates a deployment strategy.

        :param deployment_strategy_id: The deployment strategy ID.
        :param description: A description of the deployment strategy.
        :param deployment_duration_in_minutes: Total amount of time for a deployment to last.
        :param final_bake_time_in_minutes: The amount of time that AppConfig monitors for alarms before considering
        the deployment to be complete and no longer eligible for automatic
        rollback.
        :param growth_factor: The percentage of targets to receive a deployed configuration during
        each interval.
        :param growth_type: The algorithm used to define how percentage grows over time.
        :returns: DeploymentStrategy
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UpdateEnvironment")
    def update_environment(
        self,
        context: RequestContext,
        application_id: Id,
        environment_id: Id,
        name: Name = None,
        description: Description = None,
        monitors: MonitorList = None,
    ) -> Environment:
        """Updates an environment.

        :param application_id: The application ID.
        :param environment_id: The environment ID.
        :param name: The name of the environment.
        :param description: A description of the environment.
        :param monitors: Amazon CloudWatch alarms to monitor during the deployment process.
        :returns: Environment
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UpdateExtension")
    def update_extension(
        self,
        context: RequestContext,
        extension_identifier: Identifier,
        description: Description = None,
        actions: ActionsMap = None,
        parameters: ParameterMap = None,
        version_number: Integer = None,
    ) -> Extension:
        """Updates an AppConfig extension. For more information about extensions,
        see `Working with AppConfig
        extensions <https://docs.aws.amazon.com/appconfig/latest/userguide/working-with-appconfig-extensions.html>`__
        in the *AppConfig User Guide*.

        :param extension_identifier: The name, the ID, or the Amazon Resource Name (ARN) of the extension.
        :param description: Information about the extension.
        :param actions: The actions defined in the extension.
        :param parameters: One or more parameters for the actions called by the extension.
        :param version_number: The extension version number.
        :returns: Extension
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises ConflictException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("UpdateExtensionAssociation")
    def update_extension_association(
        self,
        context: RequestContext,
        extension_association_id: Id,
        parameters: ParameterValueMap = None,
    ) -> ExtensionAssociation:
        """Updates an association. For more information about extensions and
        associations, see `Working with AppConfig
        extensions <https://docs.aws.amazon.com/appconfig/latest/userguide/working-with-appconfig-extensions.html>`__
        in the *AppConfig User Guide*.

        :param extension_association_id: The system-generated ID for the association.
        :param parameters: The parameter names and values defined in the extension.
        :returns: ExtensionAssociation
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError

    @handler("ValidateConfiguration")
    def validate_configuration(
        self,
        context: RequestContext,
        application_id: Id,
        configuration_profile_id: Id,
        configuration_version: Version,
    ) -> None:
        """Uses the validators in a configuration profile to validate a
        configuration.

        :param application_id: The application ID.
        :param configuration_profile_id: The configuration profile ID.
        :param configuration_version: The version of the configuration to validate.
        :raises BadRequestException:
        :raises ResourceNotFoundException:
        :raises InternalServerException:
        """
        raise NotImplementedError



import logging,os
from typing import Dict,Optional
from localstack import config
from localstack.utils.files import cp_r,rm_rf
from plugin import Plugin
LOG=logging.getLogger(__name__)
class StateLifecyclePlugin(Plugin):
	namespace='localstack.state.lifecycle';service:str
	def get_assets_location(A):B=config.dirs.data;return os.path.join(B,A.service)
	def has_assets(B):
		A=B.get_assets_location()
		if os.path.exists(A)and len(os.listdir(A))>0:return True
		return False
	def retrieve_assets(G):
		A=G.get_assets_location();B={}
		if not os.path.isdir(A):return B
		for (D,L,H) in os.walk(A,topdown=True):
			for C in H:
				E=os.path.relpath(D,A);I=os.path.join(E,C)if E!='.'else C;F=os.path.join(D,C)
				if os.path.isfile(F):J=I;K=_load_asset_binary(F);B[J]=K
		return B
	def inject_assets(A,pod_asset_directory):
		B=A.get_assets_location();C=os.path.join(pod_asset_directory,A.service);rm_rf(B)
		if os.path.exists(C):cp_r(C,B)
	def reset_state(A):A.on_after_reset()
	def on_after_reset(A):0
def _load_asset_binary(file_path):
	A=file_path
	try:
		with open(A,'rb')as B:return B.read()
	except Exception as C:LOG.warning(f"Could not load assets binary for file {A} due to {C}.");return None


_A='global'
import logging,os
from typing import Dict,Optional,Set
from localstack import config
from localstack.aws.accounts import get_aws_account_id
from localstack.services.stores import AccountRegionBundle
from localstack.services.visitors import ReflectionStateLocator,ServiceBackend,ServiceBackendCollectorVisitor
from localstack.utils.aws.aws_stack import get_valid_regions_for_service
from localstack.utils.files import load_file
from localstack.utils.objects import singleton_factory
from localstack.utils.strings import to_bytes
from moto.core import BackendDict
from moto.s3.models import S3Backend
from plugin import PluginManager
from localstack_ext.bootstrap.pods.server.plugins import StateLifecyclePlugin
from localstack_ext.bootstrap.pods.server.visitors import PodStateLoaderVisitor
from localstack_ext.bootstrap.pods.service_state.service_state import ServiceState
from localstack_ext.bootstrap.pods.service_state.service_state_types import AccountRegion,BackendState,ServiceKey
from localstack_ext.constants import STORE_STATE_FILE
LOG=logging.getLogger(__name__)
PLUGIN_NAMESPACE='localstack.state.lifecycle'
class PersistencePluginManager(PluginManager[StateLifecyclePlugin]):
	def __init__(A):super().__init__(PLUGIN_NAMESPACE)
	@staticmethod
	@singleton_factory
	def get():return PersistencePluginManager()
	def get_plugin(B,service):
		A=service
		try:return B.load(A)
		except ValueError as C:LOG.debug('Failed to load persistence plugin for "%s": "%s',A,C)
class ServiceStateManager:
	def __init__(A,service):A.service=service;A.manager=PersistencePluginManager.get();A.manager.load_all()
	def get_service_state(A):
		D=ServiceBackendCollectorVisitor();G=ReflectionStateLocator(service=A.service);G.accept(visitor=D);E=D.collect();B=ServiceState()
		for F in E.keys():B.put_service_state(_service_state_from_backend(backend=E[F],api=A.service,memory_management=F))
		C=A.manager.get_plugin(A.service)
		if C and C.has_assets():H=C.retrieve_assets();B.put_assets(A.service,H)
		return B
	def inject_service_state(A,pod_state,pod_tmp_dir=None):
		from localstack.services.plugins import SERVICE_PLUGINS as E;F=PodStateLoaderVisitor(pod_state=pod_state,service=A.service);G=ReflectionStateLocator(service=A.service);G.accept(visitor=F);B=A.manager.get_plugin(A.service)
		if B:B.inject_assets(pod_asset_directory=pod_tmp_dir)
		C=E.get_service_container(A.service)
		if not C:LOG.debug('No service container for %s',A.service);return
		D=C.service.lifecycle_hook
		if D:
			try:D.on_after_inject()
			except Exception as H:LOG.debug('An error occurred while executing the after inject hook for %s: %s',A.service,H)
	def active_service_regions(B):
		A=B._active_service_regions()
		if B.service=='stepfunctions':
			C=os.path.join(config.dirs.data,B.service,'backend_state')
			if os.path.exists(C):
				try:D=load_file(C,mode='rb');A=[B for B in A if to_bytes(B.region)in D]
				except Exception:pass
			return set(A)
		return A
	def _active_service_regions(D):
		def B(reg):return AccountRegion(account_id=get_aws_account_id(),region=reg)
		def G(account_id):
			A=account_id
			if A=='123456789012':return get_aws_account_id()
			return A
		E=ServiceBackendCollectorVisitor();H=ReflectionStateLocator(service=D.service);H.accept(visitor=E);F=E.collect()
		if not F:I=get_valid_regions_for_service(D.service);return set([B(A)for A in I])
		C=set()
		for (M,A) in F.items():
			if isinstance(A,(BackendDict,AccountRegionBundle)):
				for (J,K) in A.items():
					for L in K.keys():C.add(AccountRegion(account_id=G(J),region=L))
			elif isinstance(A,dict):C.update(set([B(C)for C in A.keys()]))
			else:return{B(_A)}
		return C
def _service_state_from_account_region_bundle(service_backend,api):
	A=ServiceState()
	for (B,C) in service_backend.items():
		for (D,E) in C.items():F=ServiceKey(account_id=B,region=D,service=api);G=BackendState(F,{STORE_STATE_FILE:E});A.put_backend(G)
	return A
def _service_state_from_backend_state(service_backend,api):
	A=service_backend;from localstack_ext.constants import MOTO_BACKEND_STATE_FILE as D;B=ServiceState()
	if not isinstance(A,dict):
		if isinstance(A,S3Backend):
			if not A.buckets:return B
		A={_A:A}
	for (C,E) in A.items():
		if not C:continue
		for (F,G) in E.items():H=ServiceKey(account_id=C,region=F,service=api);I=BackendState(H,{D:G});B.put_backend(I)
	return B
def _service_state_from_backend(backend,api,memory_management):
	C=memory_management;B=api;A=backend;LOG.debug('Backend for service %s, memory %s, of type %s',B,C,type(A))
	if C=='localstack':return _service_state_from_account_region_bundle(service_backend=A,api=B)
	if C=='moto':return _service_state_from_backend_state(service_backend=A,api=B)


import sys
from datetime import datetime
from typing import List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

ContainerARN = str
ContainerAccessLoggingEnabled = bool
ContainerListLimit = int
ContainerName = str
ContainerPolicy = str
Endpoint = str
ErrorMessage = str
Header = str
LifecyclePolicy = str
MaxAgeSeconds = int
ObjectGroup = str
ObjectGroupName = str
Origin = str
PaginationToken = str
TagKey = str
TagValue = str


class ContainerLevelMetrics(str):
    ENABLED = "ENABLED"
    DISABLED = "DISABLED"


class ContainerStatus(str):
    ACTIVE = "ACTIVE"
    CREATING = "CREATING"
    DELETING = "DELETING"


class MethodName(str):
    PUT = "PUT"
    GET = "GET"
    DELETE = "DELETE"
    HEAD = "HEAD"


class ContainerInUseException(ServiceException):
    """The container that you specified in the request already exists or is
    being updated.
    """

    code: str = "ContainerInUseException"
    sender_fault: bool = False
    status_code: int = 400


class ContainerNotFoundException(ServiceException):
    """The container that you specified in the request does not exist."""

    code: str = "ContainerNotFoundException"
    sender_fault: bool = False
    status_code: int = 400


class CorsPolicyNotFoundException(ServiceException):
    """The CORS policy that you specified in the request does not exist."""

    code: str = "CorsPolicyNotFoundException"
    sender_fault: bool = False
    status_code: int = 400


class InternalServerError(ServiceException):
    """The service is temporarily unavailable."""

    code: str = "InternalServerError"
    sender_fault: bool = False
    status_code: int = 400


class LimitExceededException(ServiceException):
    """A service limit has been exceeded."""

    code: str = "LimitExceededException"
    sender_fault: bool = False
    status_code: int = 400


class PolicyNotFoundException(ServiceException):
    """The policy that you specified in the request does not exist."""

    code: str = "PolicyNotFoundException"
    sender_fault: bool = False
    status_code: int = 400


AllowedHeaders = List[Header]
AllowedMethods = List[MethodName]
AllowedOrigins = List[Origin]
TimeStamp = datetime


class Container(TypedDict, total=False):
    """This section describes operations that you can perform on an AWS
    Elemental MediaStore container.
    """

    Endpoint: Optional[Endpoint]
    CreationTime: Optional[TimeStamp]
    ARN: Optional[ContainerARN]
    Name: Optional[ContainerName]
    Status: Optional[ContainerStatus]
    AccessLoggingEnabled: Optional[ContainerAccessLoggingEnabled]


ContainerList = List[Container]
ExposeHeaders = List[Header]


class CorsRule(TypedDict, total=False):
    """A rule for a CORS policy. You can add up to 100 rules to a CORS policy.
    If more than one rule applies, the service uses the first applicable
    rule listed.
    """

    AllowedOrigins: AllowedOrigins
    AllowedMethods: Optional[AllowedMethods]
    AllowedHeaders: AllowedHeaders
    MaxAgeSeconds: Optional[MaxAgeSeconds]
    ExposeHeaders: Optional[ExposeHeaders]


CorsPolicy = List[CorsRule]


class Tag(TypedDict, total=False):
    """A collection of tags associated with a container. Each tag consists of a
    key:value pair, which can be anything you define. Typically, the tag key
    represents a category (such as "environment") and the tag value
    represents a specific value within that category (such as "test,"
    "development," or "production"). You can add up to 50 tags to each
    container. For more information about tagging, including naming and
    usage conventions, see `Tagging Resources in
    MediaStore <https://docs.aws.amazon.com/mediastore/latest/ug/tagging.html>`__.
    """

    Key: TagKey
    Value: Optional[TagValue]


TagList = List[Tag]


class CreateContainerInput(ServiceRequest):
    ContainerName: ContainerName
    Tags: Optional[TagList]


class CreateContainerOutput(TypedDict, total=False):
    Container: Container


class DeleteContainerInput(ServiceRequest):
    ContainerName: ContainerName


class DeleteContainerOutput(TypedDict, total=False):
    pass


class DeleteContainerPolicyInput(ServiceRequest):
    ContainerName: ContainerName


class DeleteContainerPolicyOutput(TypedDict, total=False):
    pass


class DeleteCorsPolicyInput(ServiceRequest):
    ContainerName: ContainerName


class DeleteCorsPolicyOutput(TypedDict, total=False):
    pass


class DeleteLifecyclePolicyInput(ServiceRequest):
    ContainerName: ContainerName


class DeleteLifecyclePolicyOutput(TypedDict, total=False):
    pass


class DeleteMetricPolicyInput(ServiceRequest):
    ContainerName: ContainerName


class DeleteMetricPolicyOutput(TypedDict, total=False):
    pass


class DescribeContainerInput(ServiceRequest):
    ContainerName: Optional[ContainerName]


class DescribeContainerOutput(TypedDict, total=False):
    Container: Optional[Container]


class GetContainerPolicyInput(ServiceRequest):
    ContainerName: ContainerName


class GetContainerPolicyOutput(TypedDict, total=False):
    Policy: ContainerPolicy


class GetCorsPolicyInput(ServiceRequest):
    ContainerName: ContainerName


class GetCorsPolicyOutput(TypedDict, total=False):
    CorsPolicy: CorsPolicy


class GetLifecyclePolicyInput(ServiceRequest):
    ContainerName: ContainerName


class GetLifecyclePolicyOutput(TypedDict, total=False):
    LifecyclePolicy: LifecyclePolicy


class GetMetricPolicyInput(ServiceRequest):
    ContainerName: ContainerName


class MetricPolicyRule(TypedDict, total=False):
    """A setting that enables metrics at the object level. Each rule contains
    an object group and an object group name. If the policy includes the
    MetricPolicyRules parameter, you must include at least one rule. Each
    metric policy can include up to five rules by default. You can also
    `request a quota
    increase <https://console.aws.amazon.com/servicequotas/home?region=us-east-1#!/services/mediastore/quotas>`__
    to allow up to 300 rules per policy.
    """

    ObjectGroup: ObjectGroup
    ObjectGroupName: ObjectGroupName


MetricPolicyRules = List[MetricPolicyRule]


class MetricPolicy(TypedDict, total=False):
    """The metric policy that is associated with the container. A metric policy
    allows AWS Elemental MediaStore to send metrics to Amazon CloudWatch. In
    the policy, you must indicate whether you want MediaStore to send
    container-level metrics. You can also include rules to define groups of
    objects that you want MediaStore to send object-level metrics for.

    To view examples of how to construct a metric policy for your use case,
    see `Example Metric
    Policies <https://docs.aws.amazon.com/mediastore/latest/ug/policies-metric-examples.html>`__.
    """

    ContainerLevelMetrics: ContainerLevelMetrics
    MetricPolicyRules: Optional[MetricPolicyRules]


class GetMetricPolicyOutput(TypedDict, total=False):
    MetricPolicy: MetricPolicy


class ListContainersInput(ServiceRequest):
    NextToken: Optional[PaginationToken]
    MaxResults: Optional[ContainerListLimit]


class ListContainersOutput(TypedDict, total=False):
    Containers: ContainerList
    NextToken: Optional[PaginationToken]


class ListTagsForResourceInput(ServiceRequest):
    Resource: ContainerARN


class ListTagsForResourceOutput(TypedDict, total=False):
    Tags: Optional[TagList]


class PutContainerPolicyInput(ServiceRequest):
    ContainerName: ContainerName
    Policy: ContainerPolicy


class PutContainerPolicyOutput(TypedDict, total=False):
    pass


class PutCorsPolicyInput(ServiceRequest):
    ContainerName: ContainerName
    CorsPolicy: CorsPolicy


class PutCorsPolicyOutput(TypedDict, total=False):
    pass


class PutLifecyclePolicyInput(ServiceRequest):
    ContainerName: ContainerName
    LifecyclePolicy: LifecyclePolicy


class PutLifecyclePolicyOutput(TypedDict, total=False):
    pass


class PutMetricPolicyInput(ServiceRequest):
    ContainerName: ContainerName
    MetricPolicy: MetricPolicy


class PutMetricPolicyOutput(TypedDict, total=False):
    pass


class StartAccessLoggingInput(ServiceRequest):
    ContainerName: ContainerName


class StartAccessLoggingOutput(TypedDict, total=False):
    pass


class StopAccessLoggingInput(ServiceRequest):
    ContainerName: ContainerName


class StopAccessLoggingOutput(TypedDict, total=False):
    pass


TagKeyList = List[TagKey]


class TagResourceInput(ServiceRequest):
    Resource: ContainerARN
    Tags: TagList


class TagResourceOutput(TypedDict, total=False):
    pass


class UntagResourceInput(ServiceRequest):
    Resource: ContainerARN
    TagKeys: TagKeyList


class UntagResourceOutput(TypedDict, total=False):
    pass


class MediastoreApi:

    service = "mediastore"
    version = "2017-09-01"

    @handler("CreateContainer")
    def create_container(
        self, context: RequestContext, container_name: ContainerName, tags: TagList = None
    ) -> CreateContainerOutput:
        """Creates a storage container to hold objects. A container is similar to a
        bucket in the Amazon S3 service.

        :param container_name: The name for the container.
        :param tags: An array of key:value pairs that you define.
        :returns: CreateContainerOutput
        :raises ContainerInUseException:
        :raises LimitExceededException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("DeleteContainer")
    def delete_container(
        self, context: RequestContext, container_name: ContainerName
    ) -> DeleteContainerOutput:
        """Deletes the specified container. Before you make a ``DeleteContainer``
        request, delete any objects in the container or in any folders in the
        container. You can delete only empty containers.

        :param container_name: The name of the container to delete.
        :returns: DeleteContainerOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("DeleteContainerPolicy")
    def delete_container_policy(
        self, context: RequestContext, container_name: ContainerName
    ) -> DeleteContainerPolicyOutput:
        """Deletes the access policy that is associated with the specified
        container.

        :param container_name: The name of the container that holds the policy.
        :returns: DeleteContainerPolicyOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises PolicyNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("DeleteCorsPolicy")
    def delete_cors_policy(
        self, context: RequestContext, container_name: ContainerName
    ) -> DeleteCorsPolicyOutput:
        """Deletes the cross-origin resource sharing (CORS) configuration
        information that is set for the container.

        To use this operation, you must have permission to perform the
        ``MediaStore:DeleteCorsPolicy`` action. The container owner has this
        permission by default and can grant this permission to others.

        :param container_name: The name of the container to remove the policy from.
        :returns: DeleteCorsPolicyOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises CorsPolicyNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("DeleteLifecyclePolicy")
    def delete_lifecycle_policy(
        self, context: RequestContext, container_name: ContainerName
    ) -> DeleteLifecyclePolicyOutput:
        """Removes an object lifecycle policy from a container. It takes up to 20
        minutes for the change to take effect.

        :param container_name: The name of the container that holds the object lifecycle policy.
        :returns: DeleteLifecyclePolicyOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises PolicyNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("DeleteMetricPolicy")
    def delete_metric_policy(
        self, context: RequestContext, container_name: ContainerName
    ) -> DeleteMetricPolicyOutput:
        """Deletes the metric policy that is associated with the specified
        container. If there is no metric policy associated with the container,
        MediaStore doesn't send metrics to CloudWatch.

        :param container_name: The name of the container that is associated with the metric policy that
        you want to delete.
        :returns: DeleteMetricPolicyOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises PolicyNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("DescribeContainer")
    def describe_container(
        self, context: RequestContext, container_name: ContainerName = None
    ) -> DescribeContainerOutput:
        """Retrieves the properties of the requested container. This request is
        commonly used to retrieve the endpoint of a container. An endpoint is a
        value assigned by the service when a new container is created. A
        container's endpoint does not change after it has been assigned. The
        ``DescribeContainer`` request returns a single ``Container`` object
        based on ``ContainerName``. To return all ``Container`` objects that are
        associated with a specified AWS account, use ListContainers.

        :param container_name: The name of the container to query.
        :returns: DescribeContainerOutput
        :raises ContainerNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("GetContainerPolicy")
    def get_container_policy(
        self, context: RequestContext, container_name: ContainerName
    ) -> GetContainerPolicyOutput:
        """Retrieves the access policy for the specified container. For information
        about the data that is included in an access policy, see the `AWS
        Identity and Access Management User
        Guide <https://aws.amazon.com/documentation/iam/>`__.

        :param container_name: The name of the container.
        :returns: GetContainerPolicyOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises PolicyNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("GetCorsPolicy")
    def get_cors_policy(
        self, context: RequestContext, container_name: ContainerName
    ) -> GetCorsPolicyOutput:
        """Returns the cross-origin resource sharing (CORS) configuration
        information that is set for the container.

        To use this operation, you must have permission to perform the
        ``MediaStore:GetCorsPolicy`` action. By default, the container owner has
        this permission and can grant it to others.

        :param container_name: The name of the container that the policy is assigned to.
        :returns: GetCorsPolicyOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises CorsPolicyNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("GetLifecyclePolicy")
    def get_lifecycle_policy(
        self, context: RequestContext, container_name: ContainerName
    ) -> GetLifecyclePolicyOutput:
        """Retrieves the object lifecycle policy that is assigned to a container.

        :param container_name: The name of the container that the object lifecycle policy is assigned
        to.
        :returns: GetLifecyclePolicyOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises PolicyNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("GetMetricPolicy")
    def get_metric_policy(
        self, context: RequestContext, container_name: ContainerName
    ) -> GetMetricPolicyOutput:
        """Returns the metric policy for the specified container.

        :param container_name: The name of the container that is associated with the metric policy.
        :returns: GetMetricPolicyOutput
        :raises ContainerNotFoundException:
        :raises PolicyNotFoundException:
        :raises ContainerInUseException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("ListContainers")
    def list_containers(
        self,
        context: RequestContext,
        next_token: PaginationToken = None,
        max_results: ContainerListLimit = None,
    ) -> ListContainersOutput:
        """Lists the properties of all containers in AWS Elemental MediaStore.

        You can query to receive all the containers in one response. Or you can
        include the ``MaxResults`` parameter to receive a limited number of
        containers in each response. In this case, the response includes a
        token. To get the next set of containers, send the command again, this
        time with the ``NextToken`` parameter (with the returned token as its
        value). The next set of responses appears, with a token if there are
        still more containers to receive.

        See also DescribeContainer, which gets the properties of one container.

        :param next_token: Only if you used ``MaxResults`` in the first command, enter the token
        (which was included in the previous response) to obtain the next set of
        containers.
        :param max_results: Enter the maximum number of containers in the response.
        :returns: ListContainersOutput
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("ListTagsForResource")
    def list_tags_for_resource(
        self, context: RequestContext, resource: ContainerARN
    ) -> ListTagsForResourceOutput:
        """Returns a list of the tags assigned to the specified container.

        :param resource: The Amazon Resource Name (ARN) for the container.
        :returns: ListTagsForResourceOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("PutContainerPolicy")
    def put_container_policy(
        self, context: RequestContext, container_name: ContainerName, policy: ContainerPolicy
    ) -> PutContainerPolicyOutput:
        """Creates an access policy for the specified container to restrict the
        users and clients that can access it. For information about the data
        that is included in an access policy, see the `AWS Identity and Access
        Management User Guide <https://aws.amazon.com/documentation/iam/>`__.

        For this release of the REST API, you can create only one policy for a
        container. If you enter ``PutContainerPolicy`` twice, the second command
        modifies the existing policy.

        :param container_name: The name of the container.
        :param policy: The contents of the policy, which includes the following:

        -  One ``Version`` tag

        -  One ``Statement`` tag that contains the standard tags for the policy.
        :returns: PutContainerPolicyOutput
        :raises ContainerNotFoundException:
        :raises ContainerInUseException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("PutCorsPolicy")
    def put_cors_policy(
        self, context: RequestContext, container_name: ContainerName, cors_policy: CorsPolicy
    ) -> PutCorsPolicyOutput:
        """Sets the cross-origin resource sharing (CORS) configuration on a
        container so that the container can service cross-origin requests. For
        example, you might want to enable a request whose origin is
        http://www.example.com to access your AWS Elemental MediaStore container
        at my.example.container.com by using the browser's XMLHttpRequest
        capability.

        To enable CORS on a container, you attach a CORS policy to the
        container. In the CORS policy, you configure rules that identify origins
        and the HTTP methods that can be executed on your container. The policy
        can contain up to 398,000 characters. You can add up to 100 rules to a
        CORS policy. If more than one rule applies, the service uses the first
        applicable rule listed.

        To learn more about CORS, see `Cross-Origin Resource Sharing (CORS) in
        AWS Elemental
        MediaStore <https://docs.aws.amazon.com/mediastore/latest/ug/cors-policy.html>`__.

        :param container_name: The name of the container that you want to assign the CORS policy to.
        :param cors_policy: The CORS policy to apply to the container.
        :returns: PutCorsPolicyOutput
        :raises ContainerNotFoundException:
        :raises ContainerInUseException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("PutLifecyclePolicy")
    def put_lifecycle_policy(
        self,
        context: RequestContext,
        container_name: ContainerName,
        lifecycle_policy: LifecyclePolicy,
    ) -> PutLifecyclePolicyOutput:
        """Writes an object lifecycle policy to a container. If the container
        already has an object lifecycle policy, the service replaces the
        existing policy with the new policy. It takes up to 20 minutes for the
        change to take effect.

        For information about how to construct an object lifecycle policy, see
        `Components of an Object Lifecycle
        Policy <https://docs.aws.amazon.com/mediastore/latest/ug/policies-object-lifecycle-components.html>`__.

        :param container_name: The name of the container that you want to assign the object lifecycle
        policy to.
        :param lifecycle_policy: The object lifecycle policy to apply to the container.
        :returns: PutLifecyclePolicyOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("PutMetricPolicy")
    def put_metric_policy(
        self, context: RequestContext, container_name: ContainerName, metric_policy: MetricPolicy
    ) -> PutMetricPolicyOutput:
        """The metric policy that you want to add to the container. A metric policy
        allows AWS Elemental MediaStore to send metrics to Amazon CloudWatch. It
        takes up to 20 minutes for the new policy to take effect.

        :param container_name: The name of the container that you want to add the metric policy to.
        :param metric_policy: The metric policy that you want to associate with the container.
        :returns: PutMetricPolicyOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("StartAccessLogging")
    def start_access_logging(
        self, context: RequestContext, container_name: ContainerName
    ) -> StartAccessLoggingOutput:
        """Starts access logging on the specified container. When you enable access
        logging on a container, MediaStore delivers access logs for objects
        stored in that container to Amazon CloudWatch Logs.

        :param container_name: The name of the container that you want to start access logging on.
        :returns: StartAccessLoggingOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("StopAccessLogging")
    def stop_access_logging(
        self, context: RequestContext, container_name: ContainerName
    ) -> StopAccessLoggingOutput:
        """Stops access logging on the specified container. When you stop access
        logging on a container, MediaStore stops sending access logs to Amazon
        CloudWatch Logs. These access logs are not saved and are not
        retrievable.

        :param container_name: The name of the container that you want to stop access logging on.
        :returns: StopAccessLoggingOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("TagResource")
    def tag_resource(
        self, context: RequestContext, resource: ContainerARN, tags: TagList
    ) -> TagResourceOutput:
        """Adds tags to the specified AWS Elemental MediaStore container. Tags are
        key:value pairs that you can associate with AWS resources. For example,
        the tag key might be "customer" and the tag value might be "companyA."
        You can specify one or more tags to add to each container. You can add
        up to 50 tags to each container. For more information about tagging,
        including naming and usage conventions, see `Tagging Resources in
        MediaStore <https://docs.aws.amazon.com/mediastore/latest/ug/tagging.html>`__.

        :param resource: The Amazon Resource Name (ARN) for the container.
        :param tags: An array of key:value pairs that you want to add to the container.
        :returns: TagResourceOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("UntagResource")
    def untag_resource(
        self, context: RequestContext, resource: ContainerARN, tag_keys: TagKeyList
    ) -> UntagResourceOutput:
        """Removes tags from the specified container. You can specify one or more
        tags to remove.

        :param resource: The Amazon Resource Name (ARN) for the container.
        :param tag_keys: A comma-separated list of keys for tags that you want to remove from the
        container.
        :returns: UntagResourceOutput
        :raises ContainerInUseException:
        :raises ContainerNotFoundException:
        :raises InternalServerError:
        """
        raise NotImplementedError



_F='url-or-name'
_E="[red]Error:[/red] Login to save to the Cloud Pod's platform"
_D='name'
_C=True
_B=False
_A=None
import argparse,sys,traceback
from typing import Dict,List,Optional,Set
from urllib.parse import urlparse
import click,requests
from click import Context
from localstack import config
from localstack.cli import console
from localstack.utils.analytics.cli import publish_invocation
from localstack_ext.bootstrap import licensing
from localstack_ext.bootstrap.pods.api_types import DEFAULT_MERGE_STRATEGY,MergeStrategy
from localstack_ext.bootstrap.pods.utils.common import is_comma_delimited_list
from localstack_ext.cli.cli_help import delete_help,list_help,load_help,save_help,versions_help
from localstack_ext.cli.click_utils import print_table
from localstack_ext.cli.cloud_pods_pretty import MergeStrategyPretty
from localstack_ext.cli.tree_view import TreeRenderer
class PodsCmdHandler(click.Group):
	def invoke(B,ctx):
		A=ctx
		try:
			D=B._is_restricted_cmd(A)
			if D and not licensing.is_logged_in():console.print('[red]Error:[/red] not logged in, please log in first');return sys.exit(1)
			return super(PodsCmdHandler,B).invoke(A)
		except Exception as C:
			if isinstance(C,click.exceptions.Exit):raise
			click.echo(f"Error: {C}")
			if A.parent and A.parent.params.get('debug'):click.echo(traceback.format_exc())
			A.exit(1)
	def _is_restricted_cmd(B,ctx):
		if ctx.protected_args!=['pull']:return _C
		A=argparse.ArgumentParser();A.add_argument('--name');C=dict(A.parse_known_args(ctx.args)[0].__dict__);D=C.get(_D)or'';return not B._is_public_community_pod(D)
	def _is_public_community_pod(A,pod_name):return'/'in pod_name
def _cloud_pod_initialized(pod_name):
	A=pod_name;from localstack_ext.bootstrap import pods_client as B
	if not B.is_initialized(pod_name=A):console.print(f"[red]Error:[/red] Could not find local CloudPods instance '{A}'");return _B
	return _C
def _is_host_reachable():
	A=_B
	try:B=requests.get(config.get_edge_url());return _C
	except requests.ConnectionError:console.print('[red]Error:[/red] Destination host unreachable.')
	return A
@click.group(name='pod',help='Manage the state of your instance via Cloud Pods.',cls=PodsCmdHandler,context_settings=dict(max_content_width=120))
def pod():0
@pod.command(name='delete',short_help='Delete a Cloud Pod',help=delete_help)
@click.argument(_D)
@publish_invocation
def cmd_pod_delete(name):
	A=name;from localstack_ext.bootstrap import pods_client as B;C=B.delete_pod(pod_name=A)
	if C:console.print(f"Successfully deleted {A}")
	else:console.print(f"[yellow]Could not delete Cloud Pod {A}[/yellow]")
@pod.command(name='save',short_help='Create a new Cloud Pod',help=save_help)
@click.argument(_F)
@click.option('-m','--message',help="Add a comment describing this Cloud Pod's version")
@click.option('-s','--services',help='Comma-delimited list of services to push in the Cloud Pod (all by default)')
@click.option('--visibility',help='Set the visibility of the Cloud Pod [`public` or `private`]. Does not create a new version')
@publish_invocation
def cmd_pod_save(url_or_name=_A,url=_A,services=_A,message=_A,visibility=_A):
	G='public';D=visibility;C=services;A=url_or_name;from localstack_ext.bootstrap import pods_client as E
	if not _is_host_reachable():return
	H=urlparse(A).scheme=='file'
	if url or H:
		F=url or A;B=E.export_pod(target=F)
		if B:console.print(f"Cloud Pods {F} successfully exported")
		return _C
	if not licensing.is_logged_in():console.print(_E);return
	if C and not is_comma_delimited_list(C):console.print('[red]Error:[/red] Input the services as a comma-delimited list');return _B
	if D:
		if D not in[G,'private']:console.print('[red]Error:[/red] Possible values for visibility are `public` and `private`');return
		B=E.set_public(pod_name=A,public=D==G)
		if B:console.print(f"Cloud Pod {A} made {D}")
		else:console.print('[red]Error:[/red] Visibility change failed')
		return
	I=[A.strip()for A in C.split(',')]if C else _A;B=E.push_state(pod_name=A,comment=message,services=I,local=_B)
	if B:console.print(f"Cloud Pod {A} successfully created")
	else:console.print(f"[red]Error:[/red] Failed to create Cloud Pod {A}")
@pod.command(name='load',short_help='Load the state of a Cloud Pod into the application runtime',help=load_help)
@click.argument(_F)
@click.option('-s','--strategy',help=MergeStrategyPretty.help_message(),default=MergeStrategyPretty.from_merge_strategy(DEFAULT_MERGE_STRATEGY))
@publish_invocation
def cmd_pod_load(url_or_name=_A,url=_A,strategy=MergeStrategyPretty.from_merge_strategy(DEFAULT_MERGE_STRATEGY)):
	B=url;A=url_or_name;from localstack_ext.bootstrap import pods_client as E
	if not _is_host_reachable():return
	D=MergeStrategyPretty.to_merge_strategy(strategy)
	if D is _A:console.print('[red]No such inject strategy[/red]');return
	def F(_result):
		if _result:console.print(f"Cloud Pod {A or B} successfully loaded")
		else:console.print(f"[red]Error:[/red] Failed to load Cloud Pod {A or B}")
	G=urlparse(A).scheme in['file','http','https','git']
	if B or G:H=B or A;C=E.import_pod(source=H,merge_strategy=D);F(C);return C
	if not licensing.is_logged_in():console.print(_E);return
	C=E.pull_state(pod_name=A,merge_strategy=D)
	if C:console.print(f"Cloud Pod {A} successfully loaded")
	else:console.print(f"[red]Error:[/red] Failed to load Cloud Pod {A}")
@pod.command(name='list',short_help='List all available Cloud Pods',help=list_help)
@click.option('--public','-p',help='List all the available public Cloud Pods',is_flag=_C,default=_B)
@publish_invocation
def cmd_pod_list_pods(public=_B):
	from localstack_ext.bootstrap import pods_client as B
	if public:C=B.list_public_pods();print_table(column_headers=['Cloud Pod'],columns=[C]);return
	A=B.list_pods()
	if not A:console.print('[yellow]No pods available[/yellow]')
	print_table(column_headers=['local/remote','Name'],columns=[['local+remote'if len(B)>1 else list(B)[0]for B in list(A.values())],list(A.keys())])
@pod.command(name='versions',short_help='List all available versions for a Cloud Pod',help=versions_help)
@click.argument(_D)
@publish_invocation
def cmd_pod_versions(name):
	from localstack_ext.bootstrap import pods_client as A
	if not _cloud_pod_initialized(pod_name=name):return
	B=A.get_version_summaries(pod_name=name);C='\n'.join(B);console.print(C)
@pod.command(name='inspect',help='Inspect the contents of a Cloud Pod')
@click.argument(_D)
@click.option('-f','--format',help='Format (curses, rich, json).',default='curses')
@publish_invocation
def cmd_pod_inspect(name,format):
	from localstack_ext.bootstrap import pods_client as B
	if not _cloud_pod_initialized(pod_name=name):return
	A=B.get_version_metamodel(pod_name=name,version=-1);C=['cloudwatch']
	for (D,E) in A.items():A[D]={A:B for(A,B)in E.items()if A not in C}
	TreeRenderer.get(format).render_tree(A)
def get_pods_community_commands():B='url';A=click.Group(name='pod',help='Manage the state of your instance via Cloud Pods',context_settings=dict(max_content_width=120));[A.add_command(B)for B in community_pods_commands];cmd_pod_save.params=list(filter(lambda x:isinstance(x,click.Argument),cmd_pod_save.params));cmd_pod_save.params[0].name=B;cmd_pod_save.short_help='Create a new Cloud Pod and saves it to disk';cmd_pod_save.help='\n    Save the current state of the LocalStack container in a Cloud Pod.\n    To export on a local path run the following command:\n\n    localstack pod save file://<path_on_disk>/<pod_name>\n\n    The output will be a <pod_name> zip file in the specified directory. This Cloud Pod instance can be restored at any\n    point in time with the load command.\n    ';cmd_pod_load.params=list(filter(lambda x:isinstance(x,click.Argument),cmd_pod_load.params));cmd_pod_load.params[0].name=B;cmd_pod_load.short_help='Load the state of a Cloud Pod into the application runtime from a given URL';cmd_pod_load.help="\n    Load a Cloud Pod into the running LocalStack container. Users can import Pods from different sources, i.e., local\n    storage or any provided HTTP URL.\n\n    Use the URL argument to specify the URL where the Cloud Pod's content is stored.\n\n    \x08\n    We support the following protocols:\n    localstack pod load file://<path_to_disk>\n    localstack pod load https://<some_url>\n    localstack pod load git://<user>/<repo>/<local_repo_path>\n\n    The latter option is merely a shortcut for:\n    \x08\n    localstack pod load https://raw.githubusercontent.com/<user>/<repo>/<branch>/<path>\n\n    Importing via a provided a given URL is available to all users.\n    ";return A
community_pods_commands=[cmd_pod_save,cmd_pod_load]


import sys
from datetime import datetime
from typing import Dict, List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

ActionDescription = str
ActionId = str
ActionParameterDescription = str
ActionParameterName = str
ActionParameterRequired = bool
ActionTargetName = str
ClientToken = str
CloudWatchLogGroupArn = str
ExceptionMessage = str
ExperimentActionDescription = str
ExperimentActionName = str
ExperimentActionParameter = str
ExperimentActionParameterName = str
ExperimentActionStartAfter = str
ExperimentActionStatusReason = str
ExperimentActionTargetName = str
ExperimentId = str
ExperimentStatusReason = str
ExperimentTargetFilterPath = str
ExperimentTargetFilterValue = str
ExperimentTargetName = str
ExperimentTargetParameterName = str
ExperimentTargetParameterValue = str
ExperimentTargetSelectionMode = str
ExperimentTemplateActionDescription = str
ExperimentTemplateActionName = str
ExperimentTemplateActionParameter = str
ExperimentTemplateActionParameterName = str
ExperimentTemplateActionStartAfter = str
ExperimentTemplateActionTargetName = str
ExperimentTemplateDescription = str
ExperimentTemplateId = str
ExperimentTemplateTargetFilterPath = str
ExperimentTemplateTargetFilterValue = str
ExperimentTemplateTargetName = str
ExperimentTemplateTargetParameterName = str
ExperimentTemplateTargetParameterValue = str
ExperimentTemplateTargetSelectionMode = str
ListActionsMaxResults = int
ListExperimentTemplatesMaxResults = int
ListExperimentsMaxResults = int
ListTargetResourceTypesMaxResults = int
LogSchemaVersion = int
NextToken = str
ResourceArn = str
RoleArn = str
S3BucketName = str
S3ObjectKey = str
StopConditionSource = str
StopConditionValue = str
TagKey = str
TagValue = str
TargetResourceTypeDescription = str
TargetResourceTypeId = str
TargetResourceTypeParameterDescription = str
TargetResourceTypeParameterName = str
TargetResourceTypeParameterRequired = bool


class ExperimentActionStatus(str):
    pending = "pending"
    initiating = "initiating"
    running = "running"
    completed = "completed"
    cancelled = "cancelled"
    stopping = "stopping"
    stopped = "stopped"
    failed = "failed"


class ExperimentStatus(str):
    pending = "pending"
    initiating = "initiating"
    running = "running"
    completed = "completed"
    stopping = "stopping"
    stopped = "stopped"
    failed = "failed"


class ConflictException(ServiceException):
    """The request could not be processed because of a conflict."""

    code: str = "ConflictException"
    sender_fault: bool = False
    status_code: int = 409


class ResourceNotFoundException(ServiceException):
    """The specified resource cannot be found."""

    code: str = "ResourceNotFoundException"
    sender_fault: bool = False
    status_code: int = 404


class ServiceQuotaExceededException(ServiceException):
    """You have exceeded your service quota."""

    code: str = "ServiceQuotaExceededException"
    sender_fault: bool = False
    status_code: int = 402


class ValidationException(ServiceException):
    """The specified input is not valid, or fails to satisfy the constraints
    for the request.
    """

    code: str = "ValidationException"
    sender_fault: bool = False
    status_code: int = 400


TagMap = Dict[TagKey, TagValue]


class ActionTarget(TypedDict, total=False):
    """Describes a target for an action."""

    resourceType: Optional[TargetResourceTypeId]


ActionTargetMap = Dict[ActionTargetName, ActionTarget]


class ActionParameter(TypedDict, total=False):
    """Describes a parameter for an action."""

    description: Optional[ActionParameterDescription]
    required: Optional[ActionParameterRequired]


ActionParameterMap = Dict[ActionParameterName, ActionParameter]


class Action(TypedDict, total=False):
    """Describes an action. For more information, see `FIS
    actions <https://docs.aws.amazon.com/fis/latest/userguide/fis-actions-reference.html>`__
    in the *Fault Injection Simulator User Guide*.
    """

    id: Optional[ActionId]
    description: Optional[ActionDescription]
    parameters: Optional[ActionParameterMap]
    targets: Optional[ActionTargetMap]
    tags: Optional[TagMap]


class ActionSummary(TypedDict, total=False):
    """Provides a summary of an action."""

    id: Optional[ActionId]
    description: Optional[ActionDescription]
    targets: Optional[ActionTargetMap]
    tags: Optional[TagMap]


ActionSummaryList = List[ActionSummary]
ExperimentTemplateActionStartAfterList = List[ExperimentTemplateActionStartAfter]
ExperimentTemplateActionTargetMap = Dict[
    ExperimentTemplateActionTargetName, ExperimentTemplateTargetName
]
ExperimentTemplateActionParameterMap = Dict[
    ExperimentTemplateActionParameterName, ExperimentTemplateActionParameter
]


class CreateExperimentTemplateActionInput(TypedDict, total=False):
    """Specifies an action for an experiment template.

    For more information, see
    `Actions <https://docs.aws.amazon.com/fis/latest/userguide/actions.html>`__
    in the *Fault Injection Simulator User Guide*.
    """

    actionId: ActionId
    description: Optional[ExperimentTemplateActionDescription]
    parameters: Optional[ExperimentTemplateActionParameterMap]
    targets: Optional[ExperimentTemplateActionTargetMap]
    startAfter: Optional[ExperimentTemplateActionStartAfterList]


CreateExperimentTemplateActionInputMap = Dict[
    ExperimentTemplateActionName, CreateExperimentTemplateActionInput
]


class ExperimentTemplateS3LogConfigurationInput(TypedDict, total=False):
    """Specifies the configuration for experiment logging to Amazon S3."""

    bucketName: S3BucketName
    prefix: Optional[S3ObjectKey]


class ExperimentTemplateCloudWatchLogsLogConfigurationInput(TypedDict, total=False):
    """Specifies the configuration for experiment logging to Amazon CloudWatch
    Logs.
    """

    logGroupArn: CloudWatchLogGroupArn


class CreateExperimentTemplateLogConfigurationInput(TypedDict, total=False):
    """Specifies the configuration for experiment logging."""

    cloudWatchLogsConfiguration: Optional[ExperimentTemplateCloudWatchLogsLogConfigurationInput]
    s3Configuration: Optional[ExperimentTemplateS3LogConfigurationInput]
    logSchemaVersion: LogSchemaVersion


ExperimentTemplateTargetParameterMap = Dict[
    ExperimentTemplateTargetParameterName, ExperimentTemplateTargetParameterValue
]
ExperimentTemplateTargetFilterValues = List[ExperimentTemplateTargetFilterValue]


class ExperimentTemplateTargetInputFilter(TypedDict, total=False):
    """Specifies a filter used for the target resource input in an experiment
    template.

    For more information, see `Resource
    filters <https://docs.aws.amazon.com/fis/latest/userguide/targets.html#target-filters>`__
    in the *Fault Injection Simulator User Guide*.
    """

    path: ExperimentTemplateTargetFilterPath
    values: ExperimentTemplateTargetFilterValues


ExperimentTemplateTargetFilterInputList = List[ExperimentTemplateTargetInputFilter]
ResourceArnList = List[ResourceArn]


class CreateExperimentTemplateTargetInput(TypedDict, total=False):
    """Specifies a target for an experiment. You must specify at least one
    Amazon Resource Name (ARN) or at least one resource tag. You cannot
    specify both ARNs and tags.

    For more information, see
    `Targets <https://docs.aws.amazon.com/fis/latest/userguide/targets.html>`__
    in the *Fault Injection Simulator User Guide*.
    """

    resourceType: TargetResourceTypeId
    resourceArns: Optional[ResourceArnList]
    resourceTags: Optional[TagMap]
    filters: Optional[ExperimentTemplateTargetFilterInputList]
    selectionMode: ExperimentTemplateTargetSelectionMode
    parameters: Optional[ExperimentTemplateTargetParameterMap]


CreateExperimentTemplateTargetInputMap = Dict[
    ExperimentTemplateTargetName, CreateExperimentTemplateTargetInput
]


class CreateExperimentTemplateStopConditionInput(TypedDict, total=False):
    """Specifies a stop condition for an experiment template."""

    source: StopConditionSource
    value: Optional[StopConditionValue]


CreateExperimentTemplateStopConditionInputList = List[CreateExperimentTemplateStopConditionInput]


class CreateExperimentTemplateRequest(ServiceRequest):
    clientToken: ClientToken
    description: ExperimentTemplateDescription
    stopConditions: CreateExperimentTemplateStopConditionInputList
    targets: Optional[CreateExperimentTemplateTargetInputMap]
    actions: CreateExperimentTemplateActionInputMap
    roleArn: RoleArn
    tags: Optional[TagMap]
    logConfiguration: Optional[CreateExperimentTemplateLogConfigurationInput]


class ExperimentTemplateS3LogConfiguration(TypedDict, total=False):
    """Describes the configuration for experiment logging to Amazon S3."""

    bucketName: Optional[S3BucketName]
    prefix: Optional[S3ObjectKey]


class ExperimentTemplateCloudWatchLogsLogConfiguration(TypedDict, total=False):
    """Describes the configuration for experiment logging to Amazon CloudWatch
    Logs.
    """

    logGroupArn: Optional[CloudWatchLogGroupArn]


class ExperimentTemplateLogConfiguration(TypedDict, total=False):
    """Describes the configuration for experiment logging."""

    cloudWatchLogsConfiguration: Optional[ExperimentTemplateCloudWatchLogsLogConfiguration]
    s3Configuration: Optional[ExperimentTemplateS3LogConfiguration]
    logSchemaVersion: Optional[LogSchemaVersion]


LastUpdateTime = datetime
CreationTime = datetime


class ExperimentTemplateStopCondition(TypedDict, total=False):
    """Describes a stop condition for an experiment template."""

    source: Optional[StopConditionSource]
    value: Optional[StopConditionValue]


ExperimentTemplateStopConditionList = List[ExperimentTemplateStopCondition]


class ExperimentTemplateAction(TypedDict, total=False):
    """Describes an action for an experiment template."""

    actionId: Optional[ActionId]
    description: Optional[ExperimentTemplateActionDescription]
    parameters: Optional[ExperimentTemplateActionParameterMap]
    targets: Optional[ExperimentTemplateActionTargetMap]
    startAfter: Optional[ExperimentTemplateActionStartAfterList]


ExperimentTemplateActionMap = Dict[ExperimentTemplateActionName, ExperimentTemplateAction]


class ExperimentTemplateTargetFilter(TypedDict, total=False):
    """Describes a filter used for the target resources in an experiment
    template.
    """

    path: Optional[ExperimentTemplateTargetFilterPath]
    values: Optional[ExperimentTemplateTargetFilterValues]


ExperimentTemplateTargetFilterList = List[ExperimentTemplateTargetFilter]


class ExperimentTemplateTarget(TypedDict, total=False):
    """Describes a target for an experiment template."""

    resourceType: Optional[TargetResourceTypeId]
    resourceArns: Optional[ResourceArnList]
    resourceTags: Optional[TagMap]
    filters: Optional[ExperimentTemplateTargetFilterList]
    selectionMode: Optional[ExperimentTemplateTargetSelectionMode]
    parameters: Optional[ExperimentTemplateTargetParameterMap]


ExperimentTemplateTargetMap = Dict[ExperimentTemplateTargetName, ExperimentTemplateTarget]


class ExperimentTemplate(TypedDict, total=False):
    """Describes an experiment template."""

    id: Optional[ExperimentTemplateId]
    description: Optional[ExperimentTemplateDescription]
    targets: Optional[ExperimentTemplateTargetMap]
    actions: Optional[ExperimentTemplateActionMap]
    stopConditions: Optional[ExperimentTemplateStopConditionList]
    creationTime: Optional[CreationTime]
    lastUpdateTime: Optional[LastUpdateTime]
    roleArn: Optional[RoleArn]
    tags: Optional[TagMap]
    logConfiguration: Optional[ExperimentTemplateLogConfiguration]


class CreateExperimentTemplateResponse(TypedDict, total=False):
    experimentTemplate: Optional[ExperimentTemplate]


class DeleteExperimentTemplateRequest(ServiceRequest):
    id: ExperimentTemplateId


class DeleteExperimentTemplateResponse(TypedDict, total=False):
    experimentTemplate: Optional[ExperimentTemplate]


class ExperimentS3LogConfiguration(TypedDict, total=False):
    """Describes the configuration for experiment logging to Amazon S3."""

    bucketName: Optional[S3BucketName]
    prefix: Optional[S3ObjectKey]


class ExperimentCloudWatchLogsLogConfiguration(TypedDict, total=False):
    """Describes the configuration for experiment logging to Amazon CloudWatch
    Logs.
    """

    logGroupArn: Optional[CloudWatchLogGroupArn]


class ExperimentLogConfiguration(TypedDict, total=False):
    """Describes the configuration for experiment logging."""

    cloudWatchLogsConfiguration: Optional[ExperimentCloudWatchLogsLogConfiguration]
    s3Configuration: Optional[ExperimentS3LogConfiguration]
    logSchemaVersion: Optional[LogSchemaVersion]


ExperimentEndTime = datetime
ExperimentStartTime = datetime


class ExperimentStopCondition(TypedDict, total=False):
    """Describes the stop condition for an experiment."""

    source: Optional[StopConditionSource]
    value: Optional[StopConditionValue]


ExperimentStopConditionList = List[ExperimentStopCondition]
ExperimentActionEndTime = datetime
ExperimentActionStartTime = datetime


class ExperimentActionState(TypedDict, total=False):
    """Describes the state of an action."""

    status: Optional[ExperimentActionStatus]
    reason: Optional[ExperimentActionStatusReason]


ExperimentActionStartAfterList = List[ExperimentActionStartAfter]
ExperimentActionTargetMap = Dict[ExperimentActionTargetName, ExperimentTargetName]
ExperimentActionParameterMap = Dict[ExperimentActionParameterName, ExperimentActionParameter]


class ExperimentAction(TypedDict, total=False):
    """Describes the action for an experiment."""

    actionId: Optional[ActionId]
    description: Optional[ExperimentActionDescription]
    parameters: Optional[ExperimentActionParameterMap]
    targets: Optional[ExperimentActionTargetMap]
    startAfter: Optional[ExperimentActionStartAfterList]
    state: Optional[ExperimentActionState]
    startTime: Optional[ExperimentActionStartTime]
    endTime: Optional[ExperimentActionEndTime]


ExperimentActionMap = Dict[ExperimentActionName, ExperimentAction]
ExperimentTargetParameterMap = Dict[ExperimentTargetParameterName, ExperimentTargetParameterValue]
ExperimentTargetFilterValues = List[ExperimentTargetFilterValue]


class ExperimentTargetFilter(TypedDict, total=False):
    """Describes a filter used for the target resources in an experiment."""

    path: Optional[ExperimentTargetFilterPath]
    values: Optional[ExperimentTargetFilterValues]


ExperimentTargetFilterList = List[ExperimentTargetFilter]


class ExperimentTarget(TypedDict, total=False):
    """Describes a target for an experiment."""

    resourceType: Optional[TargetResourceTypeId]
    resourceArns: Optional[ResourceArnList]
    resourceTags: Optional[TagMap]
    filters: Optional[ExperimentTargetFilterList]
    selectionMode: Optional[ExperimentTargetSelectionMode]
    parameters: Optional[ExperimentTargetParameterMap]


ExperimentTargetMap = Dict[ExperimentTargetName, ExperimentTarget]


class ExperimentState(TypedDict, total=False):
    """Describes the state of an experiment."""

    status: Optional[ExperimentStatus]
    reason: Optional[ExperimentStatusReason]


class Experiment(TypedDict, total=False):
    """Describes an experiment."""

    id: Optional[ExperimentId]
    experimentTemplateId: Optional[ExperimentTemplateId]
    roleArn: Optional[RoleArn]
    state: Optional[ExperimentState]
    targets: Optional[ExperimentTargetMap]
    actions: Optional[ExperimentActionMap]
    stopConditions: Optional[ExperimentStopConditionList]
    creationTime: Optional[CreationTime]
    startTime: Optional[ExperimentStartTime]
    endTime: Optional[ExperimentEndTime]
    tags: Optional[TagMap]
    logConfiguration: Optional[ExperimentLogConfiguration]


class ExperimentSummary(TypedDict, total=False):
    """Provides a summary of an experiment."""

    id: Optional[ExperimentId]
    experimentTemplateId: Optional[ExperimentTemplateId]
    state: Optional[ExperimentState]
    creationTime: Optional[CreationTime]
    tags: Optional[TagMap]


ExperimentSummaryList = List[ExperimentSummary]


class ExperimentTemplateSummary(TypedDict, total=False):
    """Provides a summary of an experiment template."""

    id: Optional[ExperimentTemplateId]
    description: Optional[ExperimentTemplateDescription]
    creationTime: Optional[CreationTime]
    lastUpdateTime: Optional[LastUpdateTime]
    tags: Optional[TagMap]


ExperimentTemplateSummaryList = List[ExperimentTemplateSummary]


class GetActionRequest(ServiceRequest):
    id: ActionId


class GetActionResponse(TypedDict, total=False):
    action: Optional[Action]


class GetExperimentRequest(ServiceRequest):
    id: ExperimentId


class GetExperimentResponse(TypedDict, total=False):
    experiment: Optional[Experiment]


class GetExperimentTemplateRequest(ServiceRequest):
    id: ExperimentTemplateId


class GetExperimentTemplateResponse(TypedDict, total=False):
    experimentTemplate: Optional[ExperimentTemplate]


class GetTargetResourceTypeRequest(ServiceRequest):
    resourceType: TargetResourceTypeId


class TargetResourceTypeParameter(TypedDict, total=False):
    """Describes the parameters for a resource type. Use parameters to
    determine which tasks are identified during target resolution.
    """

    description: Optional[TargetResourceTypeParameterDescription]
    required: Optional[TargetResourceTypeParameterRequired]


TargetResourceTypeParameterMap = Dict[TargetResourceTypeParameterName, TargetResourceTypeParameter]


class TargetResourceType(TypedDict, total=False):
    """Describes a resource type."""

    resourceType: Optional[TargetResourceTypeId]
    description: Optional[TargetResourceTypeDescription]
    parameters: Optional[TargetResourceTypeParameterMap]


class GetTargetResourceTypeResponse(TypedDict, total=False):
    targetResourceType: Optional[TargetResourceType]


class ListActionsRequest(ServiceRequest):
    maxResults: Optional[ListActionsMaxResults]
    nextToken: Optional[NextToken]


class ListActionsResponse(TypedDict, total=False):
    actions: Optional[ActionSummaryList]
    nextToken: Optional[NextToken]


class ListExperimentTemplatesRequest(ServiceRequest):
    maxResults: Optional[ListExperimentTemplatesMaxResults]
    nextToken: Optional[NextToken]


class ListExperimentTemplatesResponse(TypedDict, total=False):
    experimentTemplates: Optional[ExperimentTemplateSummaryList]
    nextToken: Optional[NextToken]


class ListExperimentsRequest(ServiceRequest):
    maxResults: Optional[ListExperimentsMaxResults]
    nextToken: Optional[NextToken]


class ListExperimentsResponse(TypedDict, total=False):
    experiments: Optional[ExperimentSummaryList]
    nextToken: Optional[NextToken]


class ListTagsForResourceRequest(ServiceRequest):
    resourceArn: ResourceArn


class ListTagsForResourceResponse(TypedDict, total=False):
    tags: Optional[TagMap]


class ListTargetResourceTypesRequest(ServiceRequest):
    maxResults: Optional[ListTargetResourceTypesMaxResults]
    nextToken: Optional[NextToken]


class TargetResourceTypeSummary(TypedDict, total=False):
    """Describes a resource type."""

    resourceType: Optional[TargetResourceTypeId]
    description: Optional[TargetResourceTypeDescription]


TargetResourceTypeSummaryList = List[TargetResourceTypeSummary]


class ListTargetResourceTypesResponse(TypedDict, total=False):
    targetResourceTypes: Optional[TargetResourceTypeSummaryList]
    nextToken: Optional[NextToken]


class StartExperimentRequest(ServiceRequest):
    clientToken: ClientToken
    experimentTemplateId: ExperimentTemplateId
    tags: Optional[TagMap]


class StartExperimentResponse(TypedDict, total=False):
    experiment: Optional[Experiment]


class StopExperimentRequest(ServiceRequest):
    id: ExperimentId


class StopExperimentResponse(TypedDict, total=False):
    experiment: Optional[Experiment]


TagKeyList = List[TagKey]


class TagResourceRequest(ServiceRequest):
    resourceArn: ResourceArn
    tags: TagMap


class TagResourceResponse(TypedDict, total=False):
    pass


class UntagResourceRequest(ServiceRequest):
    resourceArn: ResourceArn
    tagKeys: Optional[TagKeyList]


class UntagResourceResponse(TypedDict, total=False):
    pass


class UpdateExperimentTemplateActionInputItem(TypedDict, total=False):
    """Specifies an action for an experiment template."""

    actionId: Optional[ActionId]
    description: Optional[ExperimentTemplateActionDescription]
    parameters: Optional[ExperimentTemplateActionParameterMap]
    targets: Optional[ExperimentTemplateActionTargetMap]
    startAfter: Optional[ExperimentTemplateActionStartAfterList]


UpdateExperimentTemplateActionInputMap = Dict[
    ExperimentTemplateActionName, UpdateExperimentTemplateActionInputItem
]


class UpdateExperimentTemplateLogConfigurationInput(TypedDict, total=False):
    """Specifies the configuration for experiment logging."""

    cloudWatchLogsConfiguration: Optional[ExperimentTemplateCloudWatchLogsLogConfigurationInput]
    s3Configuration: Optional[ExperimentTemplateS3LogConfigurationInput]
    logSchemaVersion: Optional[LogSchemaVersion]


class UpdateExperimentTemplateTargetInput(TypedDict, total=False):
    """Specifies a target for an experiment. You must specify at least one
    Amazon Resource Name (ARN) or at least one resource tag. You cannot
    specify both.
    """

    resourceType: TargetResourceTypeId
    resourceArns: Optional[ResourceArnList]
    resourceTags: Optional[TagMap]
    filters: Optional[ExperimentTemplateTargetFilterInputList]
    selectionMode: ExperimentTemplateTargetSelectionMode
    parameters: Optional[ExperimentTemplateTargetParameterMap]


UpdateExperimentTemplateTargetInputMap = Dict[
    ExperimentTemplateTargetName, UpdateExperimentTemplateTargetInput
]


class UpdateExperimentTemplateStopConditionInput(TypedDict, total=False):
    """Specifies a stop condition for an experiment. You can define a stop
    condition as a CloudWatch alarm.
    """

    source: StopConditionSource
    value: Optional[StopConditionValue]


UpdateExperimentTemplateStopConditionInputList = List[UpdateExperimentTemplateStopConditionInput]


class UpdateExperimentTemplateRequest(ServiceRequest):
    id: ExperimentTemplateId
    description: Optional[ExperimentTemplateDescription]
    stopConditions: Optional[UpdateExperimentTemplateStopConditionInputList]
    targets: Optional[UpdateExperimentTemplateTargetInputMap]
    actions: Optional[UpdateExperimentTemplateActionInputMap]
    roleArn: Optional[RoleArn]
    logConfiguration: Optional[UpdateExperimentTemplateLogConfigurationInput]


class UpdateExperimentTemplateResponse(TypedDict, total=False):
    experimentTemplate: Optional[ExperimentTemplate]


class FisApi:

    service = "fis"
    version = "2020-12-01"

    @handler("CreateExperimentTemplate")
    def create_experiment_template(
        self,
        context: RequestContext,
        client_token: ClientToken,
        description: ExperimentTemplateDescription,
        stop_conditions: CreateExperimentTemplateStopConditionInputList,
        actions: CreateExperimentTemplateActionInputMap,
        role_arn: RoleArn,
        targets: CreateExperimentTemplateTargetInputMap = None,
        tags: TagMap = None,
        log_configuration: CreateExperimentTemplateLogConfigurationInput = None,
    ) -> CreateExperimentTemplateResponse:
        """Creates an experiment template.

        An experiment template includes the following components:

        -  **Targets**: A target can be a specific resource in your Amazon Web
           Services environment, or one or more resources that match criteria
           that you specify, for example, resources that have specific tags.

        -  **Actions**: The actions to carry out on the target. You can specify
           multiple actions, the duration of each action, and when to start each
           action during an experiment.

        -  **Stop conditions**: If a stop condition is triggered while an
           experiment is running, the experiment is automatically stopped. You
           can define a stop condition as a CloudWatch alarm.

        For more information, see `Experiment
        templates <https://docs.aws.amazon.com/fis/latest/userguide/experiment-templates.html>`__
        in the *Fault Injection Simulator User Guide*.

        :param client_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :param description: A description for the experiment template.
        :param stop_conditions: The stop conditions.
        :param actions: The actions for the experiment.
        :param role_arn: The Amazon Resource Name (ARN) of an IAM role that grants the FIS
        service permission to perform service actions on your behalf.
        :param targets: The targets for the experiment.
        :param tags: The tags to apply to the experiment template.
        :param log_configuration: The configuration for experiment logging.
        :returns: CreateExperimentTemplateResponse
        :raises ValidationException:
        :raises ConflictException:
        :raises ResourceNotFoundException:
        :raises ServiceQuotaExceededException:
        """
        raise NotImplementedError

    @handler("DeleteExperimentTemplate")
    def delete_experiment_template(
        self, context: RequestContext, id: ExperimentTemplateId
    ) -> DeleteExperimentTemplateResponse:
        """Deletes the specified experiment template.

        :param id: The ID of the experiment template.
        :returns: DeleteExperimentTemplateResponse
        :raises ValidationException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("GetAction")
    def get_action(self, context: RequestContext, id: ActionId) -> GetActionResponse:
        """Gets information about the specified FIS action.

        :param id: The ID of the action.
        :returns: GetActionResponse
        :raises ValidationException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("GetExperiment")
    def get_experiment(self, context: RequestContext, id: ExperimentId) -> GetExperimentResponse:
        """Gets information about the specified experiment.

        :param id: The ID of the experiment.
        :returns: GetExperimentResponse
        :raises ValidationException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("GetExperimentTemplate")
    def get_experiment_template(
        self, context: RequestContext, id: ExperimentTemplateId
    ) -> GetExperimentTemplateResponse:
        """Gets information about the specified experiment template.

        :param id: The ID of the experiment template.
        :returns: GetExperimentTemplateResponse
        :raises ValidationException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("GetTargetResourceType")
    def get_target_resource_type(
        self, context: RequestContext, resource_type: TargetResourceTypeId
    ) -> GetTargetResourceTypeResponse:
        """Gets information about the specified resource type.

        :param resource_type: The resource type.
        :returns: GetTargetResourceTypeResponse
        :raises ValidationException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("ListActions")
    def list_actions(
        self,
        context: RequestContext,
        max_results: ListActionsMaxResults = None,
        next_token: NextToken = None,
    ) -> ListActionsResponse:
        """Lists the available FIS actions.

        :param max_results: The maximum number of results to return with a single call.
        :param next_token: The token for the next page of results.
        :returns: ListActionsResponse
        :raises ValidationException:
        """
        raise NotImplementedError

    @handler("ListExperimentTemplates")
    def list_experiment_templates(
        self,
        context: RequestContext,
        max_results: ListExperimentTemplatesMaxResults = None,
        next_token: NextToken = None,
    ) -> ListExperimentTemplatesResponse:
        """Lists your experiment templates.

        :param max_results: The maximum number of results to return with a single call.
        :param next_token: The token for the next page of results.
        :returns: ListExperimentTemplatesResponse
        :raises ValidationException:
        """
        raise NotImplementedError

    @handler("ListExperiments")
    def list_experiments(
        self,
        context: RequestContext,
        max_results: ListExperimentsMaxResults = None,
        next_token: NextToken = None,
    ) -> ListExperimentsResponse:
        """Lists your experiments.

        :param max_results: The maximum number of results to return with a single call.
        :param next_token: The token for the next page of results.
        :returns: ListExperimentsResponse
        :raises ValidationException:
        """
        raise NotImplementedError

    @handler("ListTagsForResource")
    def list_tags_for_resource(
        self, context: RequestContext, resource_arn: ResourceArn
    ) -> ListTagsForResourceResponse:
        """Lists the tags for the specified resource.

        :param resource_arn: The Amazon Resource Name (ARN) of the resource.
        :returns: ListTagsForResourceResponse
        """
        raise NotImplementedError

    @handler("ListTargetResourceTypes")
    def list_target_resource_types(
        self,
        context: RequestContext,
        max_results: ListTargetResourceTypesMaxResults = None,
        next_token: NextToken = None,
    ) -> ListTargetResourceTypesResponse:
        """Lists the target resource types.

        :param max_results: The maximum number of results to return with a single call.
        :param next_token: The token for the next page of results.
        :returns: ListTargetResourceTypesResponse
        :raises ValidationException:
        """
        raise NotImplementedError

    @handler("StartExperiment")
    def start_experiment(
        self,
        context: RequestContext,
        client_token: ClientToken,
        experiment_template_id: ExperimentTemplateId,
        tags: TagMap = None,
    ) -> StartExperimentResponse:
        """Starts running an experiment from the specified experiment template.

        :param client_token: Unique, case-sensitive identifier that you provide to ensure the
        idempotency of the request.
        :param experiment_template_id: The ID of the experiment template.
        :param tags: The tags to apply to the experiment.
        :returns: StartExperimentResponse
        :raises ValidationException:
        :raises ConflictException:
        :raises ResourceNotFoundException:
        :raises ServiceQuotaExceededException:
        """
        raise NotImplementedError

    @handler("StopExperiment")
    def stop_experiment(self, context: RequestContext, id: ExperimentId) -> StopExperimentResponse:
        """Stops the specified experiment.

        :param id: The ID of the experiment.
        :returns: StopExperimentResponse
        :raises ValidationException:
        :raises ResourceNotFoundException:
        """
        raise NotImplementedError

    @handler("TagResource")
    def tag_resource(
        self, context: RequestContext, resource_arn: ResourceArn, tags: TagMap
    ) -> TagResourceResponse:
        """Applies the specified tags to the specified resource.

        :param resource_arn: The Amazon Resource Name (ARN) of the resource.
        :param tags: The tags for the resource.
        :returns: TagResourceResponse
        """
        raise NotImplementedError

    @handler("UntagResource")
    def untag_resource(
        self, context: RequestContext, resource_arn: ResourceArn, tag_keys: TagKeyList = None
    ) -> UntagResourceResponse:
        """Removes the specified tags from the specified resource.

        :param resource_arn: The Amazon Resource Name (ARN) of the resource.
        :param tag_keys: The tag keys to remove.
        :returns: UntagResourceResponse
        """
        raise NotImplementedError

    @handler("UpdateExperimentTemplate")
    def update_experiment_template(
        self,
        context: RequestContext,
        id: ExperimentTemplateId,
        description: ExperimentTemplateDescription = None,
        stop_conditions: UpdateExperimentTemplateStopConditionInputList = None,
        targets: UpdateExperimentTemplateTargetInputMap = None,
        actions: UpdateExperimentTemplateActionInputMap = None,
        role_arn: RoleArn = None,
        log_configuration: UpdateExperimentTemplateLogConfigurationInput = None,
    ) -> UpdateExperimentTemplateResponse:
        """Updates the specified experiment template.

        :param id: The ID of the experiment template.
        :param description: A description for the template.
        :param stop_conditions: The stop conditions for the experiment.
        :param targets: The targets for the experiment.
        :param actions: The actions for the experiment.
        :param role_arn: The Amazon Resource Name (ARN) of an IAM role that grants the FIS
        service permission to perform service actions on your behalf.
        :param log_configuration: The configuration for experiment logging.
        :returns: UpdateExperimentTemplateResponse
        :raises ValidationException:
        :raises ResourceNotFoundException:
        :raises ServiceQuotaExceededException:
        """
        raise NotImplementedError



_A='timescaledb.so'
import os.path
from functools import cached_property
from typing import List
from localstack.packages import InstallTarget,Package,PackageInstaller
from localstack.utils.files import new_tmp_file,rm_rf,save_file
from localstack.utils.http import download
from localstack.utils.run import run
from localstack_ext.packages import OSPackageInstaller
class TimescaleDbPackage(Package):
	def __init__(A):super().__init__('TimescaleDb','2.0.1')
	def get_versions(A):return['2.0.1']
	def _get_installer(A,version):return TimescaleDbPackageInstaller(version)
class TimescaleDbPackageInstaller(OSPackageInstaller):
	def __init__(A,version):super().__init__('timescale-db',version)
	def _debian_prepare_install(E,target):
		B='/etc/apt/sources.list.d/timescaledb.list'
		if not os.path.exists(B):A=new_tmp_file();download('https://packagecloud.io/timescale/timescaledb/gpgkey',A);run(['apt-key','add',A]);rm_rf(A);C=run(['lsb_release','-cs']).strip();D=f"deb https://packagecloud.io/timescale/timescaledb/debian/ {C} main";save_file(B,D)
		super()._debian_prepare_install(target)
	def _debian_packages(A):return[f"timescaledb-2-postgresql-11='{A.version}~debian10'",f"timescaledb-2-loader-postgresql-11='{A.version}~debian10'"]
	def _debian_get_install_marker_path(A,install_dir):return os.path.join(install_dir,_A)
	def _debian_get_install_dir(A,target):return'/usr/lib/postgresql/11/lib/'
	@cached_property
	def redhat_release(self):return run(['rpm','-E','%{rhel}']).strip()
	def _redhat_prepare_install(A,target):B=f"""
[timescale_timescaledb]
name=timescale_timescaledb
baseurl=https://packagecloud.io/timescale/timescaledb/el/{A.redhat_release}/$basearch
repo_gpgcheck=1
gpgcheck=0
enabled=1
gpgkey=https://packagecloud.io/timescale/timescaledb/gpgkey
sslverify=1
sslcacert=/etc/pki/tls/certs/ca-bundle.crt
metadata_expire=300
""";save_file('/etc/yum.repos.d/timescale_timescaledb.repo',B)
	def _redhat_packages(A):return[f"timescaledb-2-postgresql-11-{A.version}-0.el{A.redhat_release}",f"timescaledb-2-loader-postgresql-11-{A.version}-0.el{A.redhat_release}"]
	def _redhat_get_install_marker_path(A,install_dir):return os.path.join(install_dir,_A)
	def _redhat_get_install_dir(A,target):return'/usr/pgsql-11/lib/'
timescaledb_package=TimescaleDbPackage()


from localstack.packages import Package
from localstack_ext.packages.core import pro_package
@pro_package(name='ecr')
def registry_package():from localstack_ext.services.ecr.packages import registry_package as A;return A


from __future__ import annotations
import hashlib,logging
from typing import Dict,List
from localstack_ext.bootstrap.pods.service_state.service_state_types import AssetByNameType,AssetNameType,AssetValueType,BackendState,ServiceKey,ServiceNameType
LOG=logging.getLogger(__name__)
class ServiceState:
	def __init__(A):A.state={};A.assets={}
	def put_service_state(A,service_state):
		B=service_state
		for C in B.state.values():A.put_backend(C)
		for (D,E) in B.assets.items():A.put_assets(D,E)
	def put_backend(B,backend_state):
		A=backend_state
		if A.key not in B.state:B.state[A.key]=A;return
		C=B.state[A.key];C.backends.update(A.backends)
	def put_asset(A,service_name,asset_name,asset_value):B=service_name;C=A.assets.get(B,{});C[asset_name]=asset_value;A.assets[B]=C
	def put_assets(A,service_name,assets_by_name):A.assets[service_name]=assets_by_name
	def is_empty(A):return len(A.state)==0 and len(A.assets)==0
	def get_services(C):
		A=set()
		for B in C.state:A.add(B.service)
		for B in C.assets:A.add(B)
		return list(A)
	def compute_hash_on_state(A):
		B=hashlib.sha1();F=sorted(A.state.keys())
		for G in F:
			C=A.state.get(G);H=sorted(C.backends.keys())
			for D in H:
				E=C.backends.get(D)
				if isinstance(E,bytes):B.update(E)
				else:LOG.debug(f"{D} is not serialized")
		return B.hexdigest()
	def __str__(A):return f"Backends: {A.state.__str__()}\nAssets: {A.assets.__str__()}"


_A='__dict__'
import logging,os
from collections import defaultdict
from typing import Any,Callable,List,OrderedDict,Set,Tuple
from localstack.utils.files import rm_rf
from localstack.utils.objects import ObjectIdHashComparator
KINESIS_DIR='kinesis'
DYNAMODB_DIR='dynamodb'
POD_KEEP='.podkeep'
LOG=logging.getLogger(__name__)
def traverse_object(obj,handler,safe=True,visited=None):
	D=safe;C=handler;B=visited;A=obj
	try:
		if isinstance(A,dict)and type(A)not in(dict,defaultdict):traverse_object(A.__dict__,handler=C,safe=D,visited=B)
		E=get_object_dict(A)
		if E is None:return
		F,B=check_already_visited(A,B)
		if F:return
		for G in E.values():traverse_object(G,handler=C,safe=D,visited=B)
		C(A)
	except Exception as H:
		if not D:raise
		LOG.warning('Unable to add missing attributes to persistence state object %s: %s',A,H)
def check_already_visited(obj,visited):
	A=visited
	if hasattr(obj,_A):
		A=A or set();B=ObjectIdHashComparator(obj)
		if B in A:return True,A
		A.add(B)
	return False,A
def check_already_visited_obj_id(obj,visited):A=visited;A=A or set();B=id(obj);C=B in A;A.add(B);return C,A
def get_object_dict(obj):
	A=obj
	if isinstance(A,dict):return A
	B=getattr(A,_A,None);return B
def is_composite_type(obj):return isinstance(obj,(dict,OrderedDict,list,set))or hasattr(obj,_A)
def api_states_traverse(api_states_path,side_effect,mutables):
	B=side_effect
	for (A,K,E) in os.walk(api_states_path):
		for C in E:
			try:F=os.path.normpath(A).split(os.sep);G,H,I=F[-3:];B(dir_name=A,fname=C,region=I,service_name=H,account_id=G,mutables=mutables)
			except Exception as J:
				D=f"Failed to apply {B.__name__} for {C} in dir {A}: {J}";LOG.warning(D)
				if LOG.isEnabledFor(logging.DEBUG):LOG.exception(D)
				continue
def cleanse_keep_files(file_path):
	for (B,D,C) in os.walk(file_path):
		for A in C:
			if A==POD_KEEP:rm_rf(os.path.join(B,A))


import os
from typing import List
from localstack.packages import InstallTarget,Package
from localstack.utils.files import new_tmp_file,rm_rf,save_file
from localstack.utils.http import download
from localstack.utils.run import run
from localstack_ext.packages import OSPackageInstaller
POSTGRES_MAJOR_VERSION_RANGE=['11','12','13']
POSTGRES_RPM_REPOSITORY='https://download.postgresql.org/pub/repos/yum/reporpms/EL-8-x86_64/pgdg-redhat-repo-latest.noarch.rpm'
REDHAT_PLPYTHON_VERSION_MAPPING={'11':'11.17','12':'12.12','13':'13.8'}
class PostgresqlPackageInstaller(OSPackageInstaller):
	def __init__(A,version):super().__init__('postgresql',version);A._debian_install_dir=os.path.join('/usr/lib/postgresql',A.version);A._debian_package_list=[f"postgresql-{A.version}",f"postgresql-plpython3-{A.version}"];A._redhat_install_dir=os.path.join(f"/usr/pgsql-{A.version}/");A._redhat_package_list=[f"postgresql{A.version}-devel",f"postgresql{A.version}-server",f"postgresql{A.version}-plpython3-{REDHAT_PLPYTHON_VERSION_MAPPING.get(A.version)}"]
	def _debian_get_install_dir(A,target):return A._debian_install_dir
	def _debian_get_install_marker_path(A,install_dir):return os.path.join(install_dir,'bin','psql')
	def _debian_packages(A):return A._debian_package_list
	def _debian_prepare_install(E,target):
		B='/etc/apt/sources.list.d/pgdg.list'
		if not os.path.exists(B):A=new_tmp_file();download('https://www.postgresql.org/media/keys/ACCC4CF8.asc',A);run(['apt-key','add',A]);rm_rf(A);C=run(['lsb_release','-cs']).strip();D=f"deb http://apt.postgresql.org/pub/repos/apt {C}-pgdg main";save_file(B,D)
		super()._debian_prepare_install(target)
	def _redhat_get_install_dir(A,target):return A._redhat_install_dir
	def _redhat_get_install_marker_path(A,install_dir):return os.path.join(install_dir,'bin','psql')
	def _redhat_packages(A):return A._redhat_package_list
	def _redhat_prepare_install(A,target):run(['dnf','install','-y',POSTGRES_RPM_REPOSITORY]);super()._redhat_prepare_install(target)
class PostgresqlPackage(Package):
	def __init__(A,default_version='11'):super().__init__(name='PostgreSQL',default_version=default_version)
	def get_versions(A):return POSTGRES_MAJOR_VERSION_RANGE
	def _get_installer(A,version):return PostgresqlPackageInstaller(version)
postgresql_package=PostgresqlPackage()


_A='java'
import glob,logging,os
from typing import List
from localstack import config
from localstack.packages import InstallTarget,Package
from localstack.utils.http import download
from localstack.utils.run import is_command_available,run
from localstack_ext.packages import OSPackageInstaller
LOG=logging.getLogger(__name__)
DEFAULT_JAVA_VERSION='11'
JAVA_VERSIONS=['8','11']
JAVA_11_HOME=os.environ.get('JAVA_11_HOME')or'/usr/lib/jvm/java-11'
JAVA_8_HOME=os.environ.get('JAVA_8_HOME')or'/usr/lib/jvm/java-8'
class JavaPackageInstaller(OSPackageInstaller):
	def __init__(A,version):super().__init__(_A,version)
	def is_installed(B):A=B.get_java_home();return A and os.path.exists(os.path.join(A,'bin',_A))
	def _debian_get_install_dir(A,target):return A._get_jvm_install_dir()
	def _debian_get_install_marker_path(A,install_dir):return os.path.join(install_dir,'bin',_A)
	def _debian_packages(A):
		if A.version==DEFAULT_JAVA_VERSION:return[]
		return[f"adoptopenjdk-{A.version}-hotspot"]
	def _debian_prepare_install(D,target):
		C='add-apt-repository';A=os.path.join(config.TMP_FOLDER,'adoptopenjdk.key');B='https://adoptopenjdk.jfrog.io/adoptopenjdk'
		if D.version!=DEFAULT_JAVA_VERSION and not os.path.exists(A):
			download(f"{B}/api/gpg/key/public",A);run(['apt-key','add',A])
			if not is_command_available(C):run(['apt','install','-y','software-properties-common'])
			run([C,'-y',f"{B}/deb/"])
		super()._debian_prepare_install(target)
	def _post_process(A,target):
		B=A._get_jvm_install_dir();C=glob.glob(f"/usr/lib/jvm/*jdk-{A.version}-*")[0]
		if not os.path.exists(B):run(['ln','-s',C,B])
	def _get_jvm_install_dir(A):return A.get_java_home()
	def get_java_home(A):
		if A.version=='8':return JAVA_8_HOME
		if A.version=='11':return JAVA_11_HOME
		return f"/usr/lib/jvm/java-{A.version}"
class JavaPackage(Package):
	def __init__(A,default_version=DEFAULT_JAVA_VERSION):super().__init__(name='Java',default_version=default_version)
	def get_versions(A):return JAVA_VERSIONS
	def _get_installer(A,version):return JavaPackageInstaller(version)
java_package=JavaPackage()